/* aica_conf.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_pem.h"

#include "ok_aica.h"

#ifndef PREFIX
#define PREFIX "."
#endif

/* ca path */
char ca_path[256];

/*-----------------------------------------
  set CA configure
-----------------------------------------*/
int config_set_capath(char *str){
	char *p,*cp;
	int err=-1;

	if((cp=strstr(str,"ca_dir"))==NULL) goto done;
	if((cp=strchr(cp,'='))==NULL) goto done;
	p=cp+1;

	if(cp=strstr(p,"\r\n")) *cp=0;
	else if(cp=strstr(p,"\n")) *cp=0;
	else goto done;
	strncpy(ca_path,p,254);

	err = 0;
done:
	return err;
}

int CA_read_config(char *fname){
	char *buf;
	int err=-1;

	if((buf=conf_get_section(fname,"[default CA]","[default CA end]"))==NULL) goto done;

	err = config_set_capath(buf);
done:
	if(buf) AIFREE(buf);
	return err;
}

/*-----------------------------------------
  set CA path
-----------------------------------------*/
int CA_set_ca_path(CA *ca,char *path){

	if((STRDUP(ca->ca_path,path))==NULL){
		OK_set_error(ERR_ST_STRDUP,ERR_LC_CA,ERR_PT_CACONF+2,NULL);
		return -1;
	}
	return 0;
}

/*-----------------------------------------
  set library install path
-----------------------------------------*/
int CA_set_lib_path(CA *ca){
	char buf[256];
#ifdef __WINDOWS__
	if(get_install_path(buf,254)==0){
		/* get actual install path */
	}else
#endif
	strncpy(buf,PREFIX,254);

	if((STRDUP(ca->lib_path,buf))==NULL){
		OK_set_error(ERR_ST_STRDUP,ERR_LC_CA,ERR_PT_CACONF+3,NULL);
		return -1;
	}
	return 0;
}
