/* aica_file.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "ok_asn1.h"
#include "ok_pem.h"
#include "ok_rsa.h"
#include "ok_x509.h"
#include "ok_pkcs.h"

#include "ok_aica.h"

/*-----------------------------------------
  read CA information
-----------------------------------------*/
CA *CA_info_read(char *path){
	unsigned char *der=NULL;
	char buf[256];
	CA *ret=NULL;

	strncpy(buf,path,240);
	strcat (buf,PATH_DELI);
	strncat(buf,"ca.cai",16);

	if((der = ASN1_read_der(buf))==NULL) goto error;

	if((ret=ASN1_CA_info(der))==NULL) goto error;

	if((STRDUP(ret->ca_path,path))==NULL){
		OK_set_error(ERR_ST_STRDUP,ERR_LC_CA,ERR_PT_CAFILE,NULL);
		goto error;
	}
	ret->der=der;
	return ret;
error:
	if(ret){CA_free(ret);der=NULL;}
	if(der){AIFREE(der);}
	return NULL;
}

/*-----------------------------------------
  write CA information
-----------------------------------------*/
int CA_info_write(CA *ca){
	char buf[256];

	if(ca->ca_path==NULL){
		OK_set_error(ERR_ST_NULLPOINTER,ERR_LC_CA,ERR_PT_CAFILE+1,NULL);
		return -1;
	}
	strncpy(buf,ca->ca_path,240);
	strcat (buf,PATH_DELI);
	strncat(buf,"ca.cai",16);

	if(ASN1_write_der(ca->der,buf)) return -1;
	printf("Update CA information.\n");
	return 0;
}

/*-----------------------------------------
  read CA pkcs#12 file
-----------------------------------------*/
int CA_p12_read(CA *ca){
	char buf[256];

	if(ca==NULL) return -1;
	if(ca->ca_path==NULL){
		OK_set_error(ERR_ST_NULLPOINTER,ERR_LC_CA,ERR_PT_CAFILE+2,NULL);
		return -1;
	}
	strncpy(buf,ca->ca_path,240);
	strcat (buf,PATH_DELI);
	strncat(buf,"ca.p12",16);

	printf("CA PKCS#12 file open \n");

	if((ca->p12=P12_read_file(buf))==NULL) goto error;
	
	if((ca->cert=P12_get_usercert(ca->p12))==NULL) goto error;

	if((ca->prvkey=P12_get_privatekey(ca->p12))==NULL) goto error;

	if(strcmp(ca->subject,ca->cert->subject)||
		strcmp(ca->issuer,ca->cert->issuer)){
		/* when creating a new CA, ca structure doesn't have
		 * any subject and issuer information. In this case,
		 * specially we return "1".
		 */
		return 1;
	}

	return 0;
error:
	return -1;
}

/*-----------------------------------------
  output CRL & certificate file
-----------------------------------------*/
int CA_write_cert(CA *ca,Cert *cert,char *fname){
	char buf[256],tmp[32];

	printf("output certificate ..");
	fflush(stdout);

	if(PEM_write_cert(cert,fname)) return -1;

	strncpy(buf,ca->ca_path,230);
	strcat (buf,PATH_DELI);
	strcat (buf,"cert");

	sprintf(tmp,"%d.cer",cert->serialNumber);
	strcat (buf,PATH_DELI);
	strncat(buf,tmp,16);

	if(PEM_write_cert(cert,buf)) return -1;
	printf(" done.\n");
	return 0;
}

int CA_write_crl(CA *ca,char *fname){
	printf("output %s ..",ca->lprof->name);
	fflush(stdout);

	if(PEM_write_crl(ca->crl,fname)) return -1;

	printf(" done.\n");
	return 0;
}
