/* aica_lcerts.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* this is Solaris2.* */
#include <sys/stat.h>

#ifdef __WINDOWS__
# include <direct.h>
# define MKDIR(dir,gra)	mkdir((dir))
#else
# define MKDIR(dir,gra)	mkdir((dir),(gra))
#endif

#include "ok_asn1.h"
#include "ok_pem.h"
#include "ok_rsa.h"
#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_tool.h"

#include "ok_aica.h"

#ifndef AICONFIG
#define AICONFIG "aica.cnf"
#endif

int get_dn_kind(char *dn);
int csv_sign_certext(CA *ca,Req *req,char *alt);
int csv_set_subject(char *sbj,CertDN *dn);
ExtGenNames* csv_othupn2gn(char *upn);
ExtGenNames* csv_ipstr2gn(char *ip);
char *csv_check_list(char *fname, int *num);
int output_prvkey(Key *prv,int snum,char *pass);
int output_pkcs12(CA *ca,Cert *req,Key *prv,char *pass);

/* req.c */
int get_key_pairs(Key **pub,Key **prv,int size,int type);

/*-----------------------------------------
  Issuing mass of certificates
-----------------------------------------*/
int CA_issue_alotof_certificates(CA *ca,char *fname){
	int	i,j,k,num,size,type,snum,pf,err,sigalgo;
	char *cp,*t,*tp,*buf,*sbj,*alt,*prof,*pass;
	char buf2[64];
	Cert *req;
	Key *pub,*prv;
	
	if((buf=csv_check_list(fname,&num))==NULL){
		if(num==0)
		    printf("error: cannot open csv file : %s\n",fname);
		else
		    printf("error: check_csv_list : %s at line %d.\n",fname,num);
		return -1;
	}

	MKDIR("./cert",0x1c0);
	MKDIR("./key",0x1c0);
	MKDIR("./req",0x1c0);
	MKDIR("./p12",0x1c0);

	i=k=0;
	tp=buf;
	do{
		err=-1;
		pub=prv=NULL;
		req=NULL;
		prof=pass=NULL;
		k++;

		if((req=Cert_new())==NULL) goto error; /* Oops! */

		/*======= begin to analize csv line =======*/
		cp=tp; t=strchr(cp,'\n'); tp=(++t);

		if(*cp == '#'){ err=0; goto end_loop; } /* comment out */

		printf("generating a certificate : no.%d\n",i);
		i++;

		if((t=strstr(cp,"\""))==NULL) break; /* get profile name */
		*t=0; prof=t+1;

		if((t=strstr(prof,"\","))==NULL) goto error; /* get serial number */
		*t=0; cp=t+2;

		if((t=strstr(cp,",\""))==NULL) goto error; /* get subject */
		*t=0; sbj=t+2;

		snum=atoi(cp);
		if((t=strstr(sbj,"\",\""))==NULL) goto error; /* get sbjAltName */
		*t=0; alt=t+3;

		if((t=strstr(alt,"\","))==NULL) goto error; /* get sbjAltName */
		*t=0; cp=t+2;

		if((t=strstr(cp,","))==NULL) goto error; /* get key algo */
		*t=0;

		if(!strcmp(cp,"rsa")){ type=KEY_RSA_PUB; sigalgo=OBJ_SIG_SHA1RSA; }
		else if(!strcmp(cp,"dsa")){ type=KEY_DSA_PUB; sigalgo=OBJ_SIG_SHA1DSA; }
		else if(!strcmp(cp,"ecdsa")){ type=KEY_ECDSA_PUB; sigalgo=OBJ_SIG_SHA1ECDSA; }
		cp=t+1;

		if((t=strstr(cp,","))==NULL) goto error; /* get key length */
		*t=0;

		size = atoi(cp);
		cp=t+1;

		if((t=strstr(cp,",\""))==NULL) goto error; /* get p12 flag */
		*t=0; pass=t+2;

		pf=atoi(cp);
		if((t=strstr(pass,"\""))==NULL) goto error; /* get password */
		*t=0;
		/*======= end to analize csv line =======*/
		
		/* serial number check */
		j = CA_can_serialNum(ca,req,snum);
		if(j>0){
			printf("error : requested serial number (%d) is already used.\n",snum);
			goto end_loop;
		}
		if(j<0) goto error; /* system error */

		/* set request subject */
		if(csv_set_subject(sbj,&req->subject_dn)){
			printf("error : bad subject format : %s\n",sbj);
			goto end_loop;
		}
		if((req->subject=Cert_subject_str(&req->subject_dn))==NULL)
			goto error; /* system error */

		if(CA_find_statstr(ca,req->subject)){
			printf("error : requested subject is already used. (%s)\n",req->subject);
			goto end_loop;
		}

		if(CA_check_policy(ca,req)){
			printf("error : requested subject isn't matched with CA policy. (%s)\n",req->subject);
			goto end_loop;
		}

		/* get certificate profile */
		if((ca->cprof=Prof_find(ca,prof))==NULL){
			printf("error : bad profile name : %s\n",prof);
			goto end_loop;
		}

		/* generate private key and public key */
		if(get_key_pairs(&pub,&prv,size,type)) goto error;

		/* save keys to file */
		if(output_prvkey(prv,snum,pass)) goto end_loop;

		/* generate certificate request */
		OK_set_cert_sig_algo(sigalgo);
		req->pubkey_algo = type;
		req->pubkey = pub;
		if((req->der = Req_toDER(req,prv,req->der,&j))==NULL){
			printf("%s\n",OK_get_errstr());
			goto end_loop;
		}

		/* save request file */
		sprintf(buf2,"req%s%07d.p10",PATH_DELI,snum);
		if(ASN1_write_der(req->der,buf2)) goto error;

		/* sign to request */
		/* req->serialNumber is set in CA_can_serialNum */
		OK_set_cert_sig_algo(ca->sigtype);
		if(Cert_dncopy(&(ca->cert->subject_dn),&(req->issuer_dn))) goto error;
		if((req->issuer= Cert_subject_str(&(req->issuer_dn)))==NULL) goto error;
		req->version=2;

		if(Sign_before_after(ca,req)){
			printf("error : cannot add time validation\n");
			goto end_loop;
		}
		if(csv_sign_certext(ca,req,alt)){
			printf("error : cannot add certificate extensions\n");
			goto end_loop;
		}

		/* free signature memory, because req->signature has CSR
		 * signature. I should free this before a certificate signature
		 * generation.
		 */
		if(req->signature){
			AIFREE(req->signature);
			req->signature = NULL;
		}
		if(req->der) AIFREE(req->der);

		if((req->der = Cert_toDER(req,ca->prvkey,NULL,&j))==NULL){
			printf("%s\n",OK_get_errstr());
			goto end_loop;
		}

		/* update ca info */
		if(CA_add_cert(ca,req)){
			printf("error : cannot add a certificate into the list\n");
			goto end_loop;
		}
		if(Prof_add_certfile(ca->cprof,req)){
			printf("error : cannot add a certificate into the list\n");
			goto end_loop;
		}
		if(!(i%30)){
			if((ca->der = CA_toDER(ca,NULL,&j))==NULL){
				printf("error :");
				goto end_loop;
			}
			if(CA_info_write(ca)) goto error;
		}


		/* output certificate file */
		sprintf(buf2,"cert%s%d.cer",PATH_DELI,snum);
		if(ASN1_write_der(req->der,buf2)) goto error;

		/* output pkcs12 file */
		if(pf){
			if(output_pkcs12(ca,req,prv,pass)) goto end_loop;
		}

		err=0;
end_loop:
		if(err) printf("line %d, continuing next process\n",k);
		if(pass) memset(pass,0,strlen(pass));

		Key_free(pub);
		Key_free(prv);
		if(req){
			req->pubkey=NULL;
			Cert_free(req);
		}
	}while(i<num);

	AIFREE(buf);
	return 0;
error:
	return -1;
}

int get_dn_kind(char *dn){
	int i=-1;
	switch(*dn){
	case 'C': i=(dn[1]=='N')?(OBJ_DIR_CN):(OBJ_DIR_C); break;
	case 'S': i=(dn[1]=='T')?(OBJ_DIR_ST):(-1); break;
	case 'L': i=OBJ_DIR_L; break;
	case 'O': i=(dn[1]=='U')?(OBJ_DIR_OU):(OBJ_DIR_O); break;
	case 'E': i=(dn[1]=='M')?(OBJ_DIR_EMAIL):(-1); break;
	default:
		OK_set_error(ERR_ST_BADPARAM,ERR_LC_CA,ERR_PT_CALCERTS+1,NULL);
		i=-1;
		break;
	}
	return i;
}

/*-----------------------------------------------------------*/
int csv_sign_certext(CA *ca,Req *req,char *alt){
	CertExt *et=NULL,*pv=NULL,*e2,*e3;
	ExtGenNames *top=NULL,*gn;
	int ok = -1;
	char *t,*n,*c;
	
	for(et=ca->cprof->ext; et ; et=et->next){
		if(et->extnID==OBJ_X509v3_SbjAltName) break;
		pv=et;
	}
	if(et){
		if(pv==NULL) ca->cprof->ext = et->next;
		else         pv->next = et->next;
	}

	/* set extension except subject alt name */
	if(Sign_certext(ca,req)) goto done;

	/* if subject alt name didn't exist in profile extension,
	 * return now
	 */
	if(et==NULL) return 0;

	/* get current subject alt name */
	if(*alt==0) alt=NULL;
	while(alt){
		n=NULL; gn=NULL;
		for(t=alt; (*t)&&(gn==NULL) ; t++){
			switch(*t){
			case 'e':
				if(!strncmp(t,"email:",6)){
					t+=6;
					if(c=strchr(t,',')){ *c=0; n=c+1; }
					if((gn=ExtGN_set_email(t))==NULL) goto done;
				}
				break;
			case 'u':
				if(!strncmp(t,"url:",4)){
					t+=4;
					if(c=strchr(t,',')){ *c=0; n=c+1; }
					if((gn=ExtGN_set_url(t))==NULL) goto done;
				}
				break;
			case 'd':
				if(!strncmp(t,"dns:",4)){
					t+=4;
					if(c=strchr(t,',')){ *c=0; n=c+1; }
					if((gn=ExtGN_set_dns(t))==NULL) goto done;
				}
				break;
			case 'i':
				if(!strncmp(t,"ip:",3)){
					t+=3;
					if(c=strchr(t,',')){ *c=0; n=c+1; }
					if((gn=csv_ipstr2gn(t))==NULL) goto done;
				}
				break;
			case 'o':
				if(!strncmp(t,"other:upn:",10)){
					t+=10;
					if(c=strchr(t,',')){ *c=0; n=c+1; }
					if((gn=csv_othupn2gn(t))==NULL) goto done;
				}
				break;
			case '<':
				if(!strncmp(t,"<e>",3)){
					t+=3;
					if(c=strstr(t,"</e>")){ *c=0; n=c+4; }
					if((gn=ExtGN_set_email(t))==NULL) goto done;
				}else if(!strncmp(t,"<url>",5)){
					t+=5;
					if(c=strstr(t,"</url>")){ *c=0; n=c+6; }
					if((gn=ExtGN_set_url(t))==NULL) goto done;
				}else if(!strncmp(t,"<dns>",5)){
					t+=5;
					if(c=strstr(t,"</dns>")){ *c=0; n=c+6; }
					if((gn=ExtGN_set_dns(t))==NULL) goto done;
				}else if(!strncmp(t,"<ip>",4)){
					t+=4;
					if(c=strstr(t,"</ip>")){ *c=0; n=c+5; }
					if((gn=csv_ipstr2gn(t))==NULL) goto done;
				}else if(!strncmp(t,"<oth:upn>",9)){
					t+=9;
					if(c=strstr(t,"</oth:upn>")){ *c=0; n=c+10; }
					if((gn=csv_othupn2gn(t))==NULL) goto done;
				}else if(!strncmp(t,"<dn>",4)){
					CertDN dn;

					t+=4;
					if(c=strstr(t,"</dn>")){ *c=0; n=c+5; }

					cert_dn_init(&dn);
					if(!csv_set_subject(t,&dn)) goto done;
					if((gn=ExtGN_set_dn(&dn))==NULL) goto done;
					cert_dn_free(&dn);
				}
				break;
			}
		}
		if(gn){
			if(top) gn->next=top;
			top=gn;
		}
		alt = n;
	}

	/* set subject alt name to certificate */
	if(top){
		if((e2=Extnew_altname(OBJ_X509v3_SbjAltName,top))==NULL) goto done;
		for(e3=req->ext; e3 ; e3=e3->next)
			if(e3->next==NULL){ e3->next=e2; break; }
	}

	ok = 0;
done:
	if(et){
		if(pv==NULL) ca->cprof->ext = et;
		else         pv->next = et;
	}
	return ok;
}

/*-----------------------------------------------------------*/
ExtGenNames* csv_ipstr2gn(char *ip){
	unsigned char buf[16];
	char *c;
	int i;

	for(i=0;i<4;i++){
		buf[i] = atoi(ip);
		if(c=strchr(ip,'.')) ip=c+1;
	}
	return ExtGN_set_ip(buf,4);
}

ExtGenNames* csv_othupn2gn(char *upn){
	OtherName *oth_nm=NULL;
	ExtGenNames *ret=NULL;
	int i;

	if((oth_nm=ExtGN_on_new())==NULL) goto done;

	oth_nm->oid = OBJ_MS_GN_UPN;
	if((STRDUP(oth_nm->oidc,"1.3.6.1.4.1.311.20.2.3"))==NULL) goto done;
	i = strlen(upn);
	if((oth_nm->name=(unsigned char*)AIMALLOC(i+4))==NULL) goto done;

	/* should be English ?? */
	if(ASN1_set_utf8(upn,oth_nm->name,&oth_nm->nlen)) goto done;

	ret = ExtGN_set_oth(oth_nm,oth_nm->nlen);
done:
	if(oth_nm) ExtGN_on_free(oth_nm);
	return ret;
}

/*-----------------------------------------------------------*/

char *csv_check_list(char *fname, int *num){
	char *ret,*cp,*tp,*t,*sbj,*alt,*prof;
	int i,j,k;

	*num = 0;
	if((ret = get_file2buf(fname,&i))==NULL) return NULL;

	/* check csv list */
	i=j=k=0;
	tp=ret;
	do{
		/* analize csv line */
		cp=tp; t=strchr(cp,'\n');
		if(t){tp=t; *t=0;}else{tp=NULL;};

		if(*cp == '#') goto end_loop; /* This is comment line */

		if((t=strstr(cp,"\""))==NULL) break; /* get profile name */
		prof=t+1;

		if((t=strstr(prof,"\","))==NULL) goto error; /* get serial number */
		cp=t+2;

		if((t=strstr(cp,",\""))==NULL) goto error; /* get subject DN */
		sbj=t+2;

		if(atoi(cp)<0) goto error;

		if((t=strstr(sbj,"\",\""))==NULL) goto error; /* get subject alt name */
		alt=t+3;

		if((t=strstr(alt,"\","))==NULL) goto error; /* get pubkey algo */
		cp=t+2;

		if(!( ((alt+2)==cp)||strstr(alt,"email:")||strstr(alt,"dns:")||
			strstr(alt,"url:")||strstr(alt,"ip:")||strstr(alt,"other:upn:")||
			strstr(alt,"<e>")||strstr(alt,"<dns>")||strstr(alt,"<url>")||
			strstr(alt,"<ip>")||strstr(alt,"<oth:upn>")||strstr(alt,"<dn>"))) goto error;

		if((t=strstr(cp,","))==NULL) goto error;

		if(!(strstr(cp,"rsa")||strstr(cp,"dsa")||strstr(cp,"ecdsa"))) goto error;
		cp=t+1;

		if((t=strstr(cp,","))==NULL) goto error;
		cp=t+1;

		j=atoi(cp);
		if((j<=0)&&(j>2048)) goto error;

		if((t=strstr(cp,",\""))==NULL) goto error;
		sbj=t+2;

		if((t=strstr(sbj,"\""))==NULL) goto error;

		k++;
end_loop:
		*tp='\n'; tp++;
		i++;
	}while(tp);

	if(k==0){ AIFREE(ret); ret=NULL; }
	*num = k;

	return ret;
error:
	OK_set_error(ERR_ST_CA_BADCSV,ERR_LC_CA,ERR_PT_CALCERTS+2,NULL);
	AIFREE(ret);
	*num = k+1;
	return NULL;
}

int csv_set_subject(char *sbj,CertDN *dn){
	int	i,j,k;
	char *t,*tp;

	if((dn->rdn[0].tagoid=get_dn_kind(sbj))<0) return -1;
	if((tp=strchr(sbj,'='))==NULL){
		OK_set_error(ERR_ST_BADFORMAT,ERR_LC_CA,ERR_PT_CALCERTS+3,NULL);
		return -1;
	}
	*tp=0; tp++; t=tp;

    for(i=0;;i++){
		if((t = strchr(t,'/'))==NULL){
			if((STRDUP(dn->rdn[i].tag,tp))==NULL) goto error;
			dn->rdn[i].derform = asn1_str_type(tp);
			break;
		}

		if((t[1]=='S')&&(t[2]=='T')&&(t[3]=='=')){
			k = OBJ_DIR_ST; j = 4;

		}else if((t[1]=='L')&&(t[2]=='=')){
			k = OBJ_DIR_L; j = 3;

		}else if((t[1]=='O')&&(t[2]=='=')){
			k = OBJ_DIR_O; j = 3;

		}else if((t[1]=='O')&&(t[2]=='U')&&(t[3]=='=')){
			k = OBJ_DIR_OU; j = 4;

		}else if((t[1]=='C')&&(t[2]=='N')&&(t[3]=='=')){
			k = OBJ_DIR_CN; j = 4;

		}else if((t[1]=='E')&&(t[2]=='M')&&(t[3]=='=')){
			k = OBJ_DIR_EMAIL; j = 4;

		}else{
			t++; i--; continue;
		}

		*t=0;
		if((STRDUP(dn->rdn[i].tag,tp))==NULL) goto error;
		dn->rdn[i].derform  = asn1_str_type(tp);
		dn->rdn[i+1].tagoid = k;
		tp=t=t+j;
	}
	dn->num = i+1;

	return 0;
error:
	OK_set_error(ERR_ST_STRDUP,ERR_LC_CA,ERR_PT_CALCERTS+3,NULL);
	return -1;
}

int output_prvkey(Key *prv,int snum,char *pass){
	char buf[256];
	
	sprintf(buf,"key%s%07d.key",PATH_DELI,snum);
	OK_set_passwd(pass);
	
	switch(prv->key_type){
	case KEY_RSA_PRV:
		if(PEM_write_rsaprv((Prvkey_RSA*)prv,buf)) goto error;
		break;
	case KEY_DSA_PRV:
		if(PEM_write_dsaprv((Prvkey_DSA*)prv,buf)) goto error;
		break;
	case KEY_ECDSA_PRV:
		if(PEM_write_ecdsaprv((Prvkey_ECDSA*)prv,buf)) goto error;
		break;
	default:
		OK_set_error(ERR_ST_UNSUPPORTED_ALGO,ERR_LC_CA,ERR_PT_CALCERTS+6,NULL);
		printf("error : unsupported key algorithm : %d\n",prv->key_type);
		goto error;
	}
	return 0;
error:
	return -1;
}

int output_pkcs12(CA *ca,Cert *req,Key *prv,char *pass){
	PKCS12 *p12;
	P12_Baggage *bg;
	char tmp[64],buf[256];
	int ok = -1;

	if((p12=P12_new())==NULL) goto done;

	if(get_dn_for_friendlyname(&req->subject_dn,tmp)) goto done;
	if(P12_add_cert(p12,req,tmp,0xff)) goto done;
	if(P12_add_key(p12,prv,tmp,0xff)) goto done;

	for(bg=ca->p12->bag; bg ;bg=bg->next)
		if(bg->type==OBJ_P12v1Bag_CERT)
			if(P12_add_cert(p12,((P12_CertBag*)bg)->cert,NULL,0xff))
				goto done;

	P12_check_chain(p12,0); /* sort pkcs12 bags */

    /* file out put */
	sprintf(buf,"p12%s%07d.p12",PATH_DELI,req->serialNumber);
	OK_set_passwd(pass);

	if(P12_write_file(p12,buf)) goto done;

	ok = 0;
done:
    /* free p12 -- be careful! bags have just pointer,
     *     but P12_free will free all cert and key memory.
     *     so I need to set NULL pointer at each cert and key...
     */
	for(bg=p12->bag; bg ;bg=bg->next)
		((P12_CertBag*)bg)->cert = NULL;	/* *key is same location as *cert */

    P12_free(p12);
	return ok;
}

