/* aica_new.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_asn1.h"
#include "ok_pem.h"
#include "ok_rsa.h"
#include "ok_x509.h"
#include "ok_pkcs.h"

#include "ok_aica.h"


/* req.c */
int get_key_pairs(Key **pub,Key **prv,int size,int type);
char *input_subject(CertDN *dn,char *prompt);

/*-----------------------------------------
  Initialize CA information
-----------------------------------------*/
int CA_init_Info(CA *ca){
	char buf[256];
	CertProf *cpf;

	strncpy(buf,ca->ca_path,210);
	strcat (buf,PATH_DELI);
	strcat (buf,"cert");	

	/***** create user certificate profile *****/
	/* smime user cert */
	if((ca->profList=Prof_cert_new(ca,"SMIME user"))==NULL) goto error;
	cpf = ca->cprof = ca->profList;
	if(Prof_cert_settmpl(ca,"SMIME user Profile template")) goto error;

	if(Prof_save_ctinfo(ca,buf)) goto error;
	if(Prof_open_ctfile(ca,buf)) goto error;

	/* cross cert */
	if((cpf->next=Prof_cert_new(ca,"Cross Cert"))==NULL) goto error;
	ca->cprof = cpf->next;
	if(Prof_cert_settmpl(ca,"Cross Cert Profile template")) goto error;
	ca->cprof->isCross = 1;

	if(Prof_save_ctinfo(ca,buf)) goto error;
	if(Prof_open_ctfile(ca,buf)) goto error;
	
	/***** create CRL profiles *****/
	/* ARL */
	if((ca->crlpList=Prof_crl_new(ca,"ARL"))==NULL) goto error;
	ca->lprof = ca->crlpList;
	if(Prof_crl_settmpl(ca,"ARL Profile template")) goto error;
	if(Prof_save_clinfo(ca,buf)) goto error;

	/* CRL */
	if((ca->lprof->next=Prof_crl_new(ca,"CRL"))==NULL) goto error;
	ca->lprof = ca->lprof->next;
	if(Prof_crl_settmpl(ca,"CRL Profile template")) goto error;
	if(Prof_save_clinfo(ca,buf)) goto error;

	/* CRL-All */
	if((ca->lprof->next=Prof_crl_new(ca,"CRL-All"))==NULL) goto error;
	ca->lprof = ca->lprof->next;
	if(Prof_crl_settmpl(ca,"CRL-All Profile template")) goto error;
	if(Prof_save_clinfo(ca,buf)) goto error;

	/* set ca subject DN policy */
	if((ca->policy = (char*)AIMALLOC(16))==NULL){
		OK_set_error(ERR_ST_MEMALLOC,ERR_LC_CA,ERR_PT_CANEW,NULL);
		goto error;
	}
	memset(ca->policy,2,16);

	return 0;
error:
	return -1;
}

int CA_new_with_p12(CA *ca){
	char *dmy="dammy subject";

	ca->subject=ca->issuer=dmy;
	if(CA_p12_read(ca)<0) goto error;
	if(Cert_is_CA(ca->cert)<0) return 1;

	ca->sigtype = ca->cert->signature_algo;
	ca->keylong = ca->cert->pubkey->size;
	if(CA_init_Info(ca)) goto error;
	if(CA_set_defaulttags(ca,&ca->cert->subject_dn)) goto error;

	return 0;
error:
	return -1;
}

int CA_new_generate(CA *ca,int size,int type){
	CertProf *cpf;
	Key *pub,*prv;
	Cert *ct;
	char *tmp;

	pub=prv=NULL;
	if((ct = Cert_new())==NULL) goto error;
	if(get_key_pairs(&pub,&prv,size,type)) goto error;

	ct->pubkey_algo   = type;
	ct->pubkey        = pub;
	ct->signature_algo= CA_key2sigalgo(type);

	ca->prvkey  = prv;
	ca->cert    = ct;
	ca->sigtype = ct->signature_algo;
	ca->keylong = size;

	if((tmp=input_subject(&ct->subject_dn,NULL))==NULL) goto error;
	FREE(tmp);
	if((ct->subject= Cert_subject_str(&(ct->subject_dn)))==NULL)
		goto error;

	if(CA_init_Info(ca)) goto error;
	if(CA_set_defaulttags(ca,&ct->subject_dn)) goto error;

	/* set temporary ca template */
	if((cpf=Prof_cert_new(ca,"CATemp"))==NULL) goto error;
	cpf->next = ca->profList;
	ca->cprof = ca->profList = cpf;
	if(Prof_cert_settmpl(ca,"Sub-CA Profile template")) goto error;
	ca->cprof->serialNum = 0;

	return 0;
error:
	Key_free(pub);
	Key_free(prv);
	if(ct){
		ct->pubkey=NULL;
		Cert_free(ct);
	}
	return -1;
}

int CA_update_P12andInfo(CA *ca,int flg){
	PKCS12 *p12;

	if((STRDUP(ca->subject,ca->cert->subject))==NULL){
		OK_set_error(ERR_ST_STRDUP,ERR_LC_CA,ERR_PT_CANEW+4,NULL);
		goto error;
	}
	if((STRDUP(ca->issuer,ca->cert->issuer))==NULL){
		OK_set_error(ERR_ST_STRDUP,ERR_LC_CA,ERR_PT_CANEW+4,NULL);
		goto error;
	}

	if(flg){
		/* 
		 * PKCS#12 file was given, so add CA certificate to the list
		 */
		ca->serialNum++;
		if(CA_add_cert(ca,ca->cert)) goto error;
		if(ASN1_write_der(ca->cert->der,"ca.cer")) goto error;
	}else{
		/*
		 * key pairs and CA certificate (self signed) was created.
		 * now generate a CA PKCS#12 file.
		 */
		if((p12=ca->p12=P12_new())==NULL) goto error;
		if(P12_add_cert(p12,ca->cert,NULL,0)) goto error;
		if(P12_add_key(p12,ca->prvkey,NULL,0)) goto error;

		if(ASN1_write_der(ca->cert->der,"ca.cer")) goto error;
		if(P12_write_file(p12,"ca.p12")) goto error;
	}

	return 0;
error:
	return -1;
}

/*-----------------------------------------
  Get signature algo from key algo
-----------------------------------------*/
int CA_set_defaulttags(CA *ca, CertDN *dir){
	char buf[256],*m_tag;
	int i,j;

	for(i=ca->sbj_num=0;i<dir->num;i++){
		switch(j=dir->rdn[i].tagoid){
		case OBJ_DIR_C: strcpy(buf,"C="); m_tag="O";  break;
		case OBJ_DIR_ST:strcpy(buf,"ST="); m_tag="O"; break;
		case OBJ_DIR_L: strcpy(buf,"L="); m_tag="O";  break;
		case OBJ_DIR_O: strcpy(buf,"O="); m_tag="CN"; break;
		case OBJ_DIR_OU:strcpy(buf,"OU="); m_tag="CN"; break;
		}
		if((j==OBJ_DIR_CN)||(j==OBJ_DIR_EMAIL)){ m_tag="CN"; break; }

		strncat(buf,dir->rdn[i].tag,250);
		if((STRDUP(ca->sbjList[i],buf))==NULL) goto error;
		ca->sbj_num++;
	}
	if((STRDUP(ca->m_tag,m_tag))==NULL) goto error;

	return 0;
error:
	OK_set_error(ERR_ST_STRDUP,ERR_LC_CA,ERR_PT_CANEW+5,NULL);
	return -1;
}
