/* aica_pol.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_asn1.h"

#include "ok_aica.h"

/*-----------------------------------------
  CA policy check with CSR
  output : 0 ... no error
         : -1... policy mismatch
-----------------------------------------*/
int CA_check_policy(CA *ca,Cert *ct){
	/* tkind is 0(C),1(ST),2(L),3(O),4(OU),5(CN),9(EMAIL) */
	int i,j,k,flag;
	int kd[] ={OBJ_DIR_C, OBJ_DIR_ST, OBJ_DIR_L, OBJ_DIR_O, 
		       OBJ_DIR_OU,OBJ_DIR_CN, OBJ_DIR_EMAIL};
	int num  = ct->subject_dn.num;
	int cnum = ca->cert->subject_dn.num;
	Cert *cert = ca->cert;

	if(ca->policy==NULL){
		OK_set_error(ERR_ST_NULLPOINTER,ERR_LC_CA,ERR_PT_CAPOL,NULL);
		return -1;
	}

	for(i=0;i<10;i++){
		if((5<i)&&(i<9)) continue;

		switch(ca->policy[i]){
		case 0:/* match */
			for(flag=k=j=0;j<cnum;j++){
				if(cert->subject_dn.rdn[j].tagoid==kd[i]){
					for(flag=1;k<num;k++){
						if(ct->subject_dn.rdn[k].tagoid==kd[i])
							if(strcmp(cert->subject_dn.rdn[j].tag,ct->subject_dn.rdn[k].tag))
								goto error;
							else{
								flag=0;
								break;
							}
					}
				}
			}
			if(flag) goto error;
			break;
		case 1:/* supplied */
			for(j=0;j<num;j++)
				if(ct->subject_dn.rdn[j].tagoid==kd[i])
					break;
			if(j==num) goto error;
			break;
		case 2:/* optional */
		default:
			break;
		}
	}

	return 0;
error:
	OK_set_error(ERR_ST_CA_POLMIS,ERR_LC_CA,ERR_PT_CAPOL,NULL);
	return -1;
}


/*-----------------------------------------
  set CA policy
-----------------------------------------*/
int CA_set_policy(CA *ca,char *in){
	int i,j;
	int kd[] ={0,1,2,3,4,5,9};

	for(i=0;i<7;i++,in++){
		switch(*in){
		case 'm':
		case 'M':
			j=0;
			break;
		case 's':
		case 'S':
			j=1;
			break;
		case 'o':
		case 'O':
			j=2;
			break;
		case ':':
			continue;
		default:
			goto error;
			break;
		}
		ca->policy[kd[i]] = j;
		in++;
		if((i<6)&&(*in != ':')) goto error;
	}
	return 0;
error:
	OK_set_error(ERR_ST_BADFORMAT,ERR_LC_CA,ERR_PT_CAPOL+1,NULL);    
	return -1;
}
