/* aica_print.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_asn1.h"

#include "ok_aica.h"

void print_ca_policy(unsigned char *pol);

/* cert_print.c */
void print_v3_extensions(CertExt *top);


void CA_print(CA *ca,char *pfname,int pfall){
	CertProf *cpf;
	CRLProf *lpf;
	int flg=0;

	printf("Issuer  :\n  %s\n",ca->issuer);
	printf("Subject :\n  %s\n",ca->subject);
	print_ca_policy(ca->policy);

	cpf=Prof_find(ca,pfname);
	lpf=Prof_crl_find(ca,pfname);

	flg = ((cpf==NULL)&&(lpf==NULL))||(pfall);
	if(flg){ cpf=ca->profList; lpf=ca->crlpList; }

	/* print cert profile */
	for(; cpf ;cpf=cpf->next){
		printf("------------------------------------\n");
		printf("Certificate Profile : %s\n",cpf->name);
		printf("  current serial number: %d\n",cpf->serialNum);
		printf("  certificate days     : %d\n",cpf->cert_days);
		print_v3_extensions(cpf->ext);
		if(!flg) break;
	}
	/* print CRL profile */
	for(; lpf ;lpf=lpf->next){
		printf("------------------------------------\n");
		printf("CRL Profile : %s\n",lpf->name);
		printf("  CRL number  : %d\n",lpf->crlNum);
		printf("  CRL days    : %d\n",lpf->crl_days);
		print_v3_extensions(lpf->ext);
		if(!flg) break;
	}
}

void print_ca_policy(unsigned char *pol){
	char cp[3][16]={"matched","supplied","option"};
	int kd[] ={0,1,2,3,4,5,9};

	printf("CA policy : ");
	printf("C=%s:",cp[pol[0]]);
	printf("ST=%s:",cp[pol[1]]);
	printf("L=%s:",cp[pol[2]]);
	printf("O=%s:",cp[pol[3]]);
	printf("OU=%s:",cp[pol[4]]);
	printf("CN=%s:",cp[pol[5]]);
	printf("Email=%s\n",cp[pol[9]]);
}

void CA_certstat_print(CA *ca,char *pfname){
	char	s[4],p1[64],*p2;
	CertStat *st;
	CertProf *cpf;

	if((cpf=Prof_find(ca,pfname))==NULL){
		printf("unknown profile...cannot list certificates\n");
		return;
	}

	printf("certificate list [profile : %s]\n",cpf->name);
	printf(" state  serial    subject             notBefore      notAfter      revokedDate\n");
	s[3]=0;
	for(st=cpf->stat; st ;st=st->next){
		(st->state&STAT_CA)?(s[0]='C'):(s[0]='-');
		(st->state&STAT_EXPIRED)?(s[1]='E'):(s[1]='-');
		(st->state&STAT_REVOKED)?(s[2]='R'):(s[2]='-');

		if((p2=stm2str(&st->notBefore,1))==NULL) p2="--";
		strcpy(p1,p2);
		if((p2=stm2str(&st->notAfter,1))==NULL)  p2="--";

		printf("  %s,  %5d, \"%s\", %s, %s, ",s,st->serialNum,st->subject,p1,p2);

		if(st->revokedDate.tm_year){
			if((p2=stm2str(&st->revokedDate,1))==NULL) p2="--";
		}else{
			p2="";
		}
		printf("%s\n",p2);
	}
}









