/* aica_prof.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef __WINDOWS__
# include <windows.h>
#else
# include <dirent.h>
#endif

#include "ok_asn1.h"
#include "ok_pem.h"
#include "ok_rsa.h"
#include "ok_x509.h"
#include "ok_pkcs.h"

#include "ok_aica.h"

typedef struct _prof {
	struct _prof* next;
	short int num;
	short int del;
	CertProf* cpf;
} PROF_t;

enum {PF_DEL,PF_ADD};

#if 0
#define DEBUG
#endif

#define MAX_PROF_TEMPLATE 32

static char PROF_TEMPLATE[MAX_PROF_TEMPLATE][256];

#ifdef DEBUG
static void
CA_dump_prof(PROF_t* top)
{
	PROF_t* p;
	for (p = top; p; p = p->next) {
		/* PRINTING ACTIVE PROFILE */
		if (!p->del)fprintf(stdout,"DUMP[%s]\n",p->cpf->name);
	}
}
#endif

/*-------------------------------------------------*/
static void
CA_free_prof(PROF_t* top)
{
	PROF_t* p;
	for (p = top; p; ) {
		PROF_t* prev = p; p = p->next; FREE(prev);
	}
}

/*-------------------------------------------------*/
static int
CA_scan_num(PROF_t* top)
{
	PROF_t* p;
	int num = 0;
	for (p = top; p; p = p->next) {
		if (!p->del) p->num = ++num;
	}
	return num;
}

/*-------------------------------------------------*/
static PROF_t*
CA_scan_prof(CA* ca)
{
	CertProf *cpf = NULL;
	int n;
	PROF_t* top = (PROF_t*)NULL;
	PROF_t** next = &top;

	for ( n = 1, cpf = ca->profList; cpf; cpf = cpf->next, n++ ) {
		PROF_t* ptr;
		
		if ((ptr=MALLOC(sizeof(PROF_t)))==NULL) goto error;

		*next = ptr;
		memset(ptr, 0x00, sizeof(PROF_t)); ptr->cpf = cpf;
		next = &ptr->next;
	}
	CA_scan_num( top );

	return top;
error:
	CA_free_prof(top);
	return NULL;
}


/*-------------------------------------------------*/
static int
CA_add_prof(CA* ca, PROF_t* top)
{
	char path[256];
	char buf [256];
	int	templates = 0;
	int loop = 0;
	int	rc = -1;

	strncpy(buf,ca->ca_path,210);
	strcat (buf,PATH_DELI);
	strcat (buf,"cert");

	if(get_tmpl_path(ca,path,216)) goto error;

#ifdef __WINDOWS__
	{
		char tbf[256],*cp;
		WIN32_FIND_DATA fname;
		HANDLE hd;

		strcpy(tbf,path);
		strcat(tbf,"*.cpi");
		hd = FindFirstFile(tbf,&fname);

		do{
			strncpy(tbf,fname.cFileName,254);
			if(cp=strstr(tbf,".cpi")){
				*cp=0;
				strncpy(PROF_TEMPLATE[templates++], tbf, 254);
			}
		}while(FindNextFile(hd,&fname));

		FindClose(hd);
	}
#else
	{
		DIR* dir;
		struct dirent* d;

		if (!(dir = opendir(path))) goto error;
		while((d = readdir(dir))) {
			char *ext, tbf[256];

			strcpy(tbf, d->d_name);
			ext = rindex(tbf, '.');
			if (ext && !strcmp(ext, ".cpi")) {
				*ext = 0x00;
				strncpy(PROF_TEMPLATE[templates++], tbf, 254);
			}
		}
		closedir(dir);
	}
#endif

	while( 1 ) {
		char inp[256];
		CertProf *cpf = (CertProf *)NULL;
		PROF_t* p;
		int	i,find=0;

		fprintf(stdout, "-------\n");
		fprintf(stdout, "Add a certificate profile to this CA.\n\n");
		for (i = 0; i < templates; i++) {
			fprintf(stdout, "[%d] %s\n", i+1, PROF_TEMPLATE[i]);
		}
		fprintf(stdout, "[0] Exit\n\n");

		if((loop=ca_ask_num("Please select a templete number",0))==0) break;

		if((loop<0)||(templates<=loop)){
			fprintf(stdout, "Invalid templete number. Please input again.\n");
			continue;
		}

		fprintf(stdout,"\n  Selected profile templete is \"%s\"\n", PROF_TEMPLATE[loop - 1]);
		if(ca_ask_comment("  Input Profile Name",inp,254));
		if (!*inp){
			fprintf(stdout, "Invalid templete name. Please input again.\n");
			continue;
		}

		for (p = top; p; p = p->next) {
			if (!p->del && !strcmp(p->cpf->name, inp)) {
				find = 1;
				break;
			}
		}
		if (find) {
			fprintf(stdout,"\nProfile Name is already used. Please input again.\n\n");
			continue;
		}

		/**** ADD PROFILE ****/
		if ((cpf=Prof_cert_new(ca, inp))==NULL) goto error;

		ca->cprof = cpf;
		if (Prof_cert_settmpl(ca,PROF_TEMPLATE[loop - 1])) goto error;
		if (Prof_save_ctinfo(ca,buf)) goto error;
		if (Prof_open_ctfile(ca,buf)) goto error;

		for (p = top; p->next; p = p->next);
		if (!(p->next = (PROF_t*)MALLOC(sizeof(PROF_t)))) goto error;
		memset(p->next, 0x00, sizeof(PROF_t));
		p->next->cpf = cpf;
	
		if(!ca_ask_yesno("\ndo you continue this operation ?",0)) break;
	}

	rc = CA_scan_num(top);

	return rc;
error:
	return -1;
}
			
/*-------------------------------------------------*/
static int
CA_del_prof(PROF_t* top)
{
	PROF_t* p;
	int i,loop = CA_scan_num(top);

	if(loop<=2){
		fprintf(stdout, "-------\n");
		fprintf(stdout, "The CA should have more than one profiles.\n");
		fprintf(stdout, "It is not available to delete any profile from this CA.\n");
		return loop;
	}

	while(loop>2){
		fprintf(stdout, "-------\n");
		fprintf(stdout, "Delete a certificate profile from this CA.\n\n");

		for (p = top; p; p = p->next) {
			if (!p->del) {
				if(strcmp(p->cpf->name,"Cross Cert")) /* should not equal Cross Cert */
					fprintf(stdout,"[%d] %s\n", p->num, p->cpf->name);
			}
		}
		fprintf(stdout, "[0] Exit\n\n");

		if((i=ca_ask_num("Please select a profile number",0))==0) break;
		if((i < 0)||(loop<i)){
			fprintf(stdout, "Invalid profile number. Please input again.\n");
			continue;
		}

		for (p = top; p; p = p->next) {
			if (!p->del && (p->num==i) &&
				strcmp(p->cpf->name,"Cross Cert")){ /* should not equal Cross Cert */
				p->del++;
				break;
			}
		}

		if(!ca_ask_yesno("do you continue this operation ?",0)) break;
		loop = CA_scan_num(top);
	}

	return loop;
}


/*-------------------------------------------------*/
static int
CA_prof_update(CA* ca,PROF_t* top)
{
	CertProf *cpf, *last;
	PROF_t* p;
	CertProf** next;
	int i;

#ifdef DEBUG
CA_dump_prof( top );
#endif

#ifdef DEBUG
for (cpf = ca->profList; cpf; cpf = cpf->next) {
	printf("OLD[%s]\n",cpf->name);
}
#endif

	next = &ca->profList; last = NULL;
	for (p = top; p; p = p->next) {
		if (!p->del) {
			*next = last = p->cpf;
			next = &p->cpf->next;
		}
	}
	if (last) last->next = NULL;

#ifdef DEBUG
for (cpf = ca->profList; cpf; cpf = cpf->next) {
	printf("NEW[%s]\n",cpf->name);
}
#endif

	if(ca->der) AIFREE(ca->der);
	if((ca->der = CA_toDER(ca,NULL,&i))==NULL) goto error;
	if(CA_info_write(ca)) goto error;

	return 0;
error:
	return -1;
}

/*--------------------------------------------------
	do profile operation.
--------------------------------------------------*/
int CA_do_profile(CA* ca, int mode)
{
	PROF_t* top = CA_scan_prof(ca);
	int rc = -1;

	switch( mode ) {
	case PF_DEL:
		if(CA_del_prof( top )<0) goto done;
		break;
	case PF_ADD:
		if(CA_add_prof(ca,top)<0) goto done;
		break;
	default: 
		return( 0 );
	}
	if(CA_prof_update(ca, top)<0) goto done;

	/*** Free Temporary Mem ***/
	CA_free_prof(top);

	rc=0;
done:
	return rc;
}
