/* aica_setext.c */
/*
 * Copyright (C) 1998-2002
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include "aiconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_asn1.h"
#include "ok_pem.h"
#include "ok_rsa.h"
#include "ok_x509.h"
#include "ok_pkcs.h"

#include "ok_aica.h"


int ca_ask_add_or_delete(){
	char buf[128] = "";
	int ret = 1;

	for(;;){
		printf("Add or delete X.509 extension for profile.\n");
		printf("Select operation (a..add/d..delete/c..cancel)[a]: ");
		fgets(buf,64,stdin);

		if((*buf==0)||(*buf=='\n')||(*buf=='a')){ ret=1; break; }
		if(*buf=='d'){ ret=0; break; }
		if(*buf=='c'){ ret=-1; break; }

		printf("Invalid operation! Please input again.\n");
	}
	return ret;
}

int ca_ask_onoff(char *txt,int on){
	char buf[128] = "";
	int ret = 1;

	for(;;){
		printf("%s (on/off)[%s]: ",txt,(on)?("on"):("off"));
		fgets(buf,64,stdin);

		if(strstr(buf,"on")){ ret=1; break; }
		if(strstr(buf,"off")){ ret=0; break; }
		if((*buf==0)||(*buf=='\n')){ ret=on; break; }

		printf("Invalid operation! Please input again.\n");
	}
	return ret;
}

int ca_ask_yesno(char *txt,int yes){
	char buf[128] = "";
	int ret = 1;

	for(;;){
		printf("%s (y/n)[%s]: ",txt,(yes)?("y"):("n"));
		fgets(buf,64,stdin);

		if(strstr(buf,"y")){ ret=1; break; }
		if(strstr(buf,"n")){ ret=0; break; }
		if((*buf==0)||(*buf=='\n')){ ret=yes; break; }

		printf("Invalid operation! Please input again.\n");
	}
	return ret;
}

int ca_ask_num(char *txt,int num){
	char buf[128] = "";
	int ret = num;

	printf("%s [%d]: ",txt,num);
	fgets(buf,64,stdin);

	if((*buf==0)||(*buf=='\n')) ret=num;
	else ret=atoi(buf);

	return ret;
}

int ca_ask_comment(char *txt,char *buf, int max){
	char *cp;
	printf("%s : ",txt);
	fgets(buf,max,stdin);
	if(cp=strchr(buf,'\n')) *cp=0;
	return 0;
}

/*--------------------------------------------
	ask extensions
--------------------------------------------*/
CertExt *ca_ask_basiccons(CA *ca){
	CertExt *ret;
	int caf=0,path=-1,crit=1;

	printf("------\n");
	printf("Configure detail information for BasicConstraints extension.\n");

	crit = ca_ask_yesno("set critical flag",1);
	caf  = ca_ask_yesno("set CA flag ",0);
	path = ca_ask_num("set pathLength (-1 means 'NULL')",-1);

	if((ret=Extnew_basic_cons(caf,path))==NULL) return NULL;
	ret->critical = crit;
	return ret;
}

CertExt *ca_ask_keyusage(CA *ca){
	CertExt *ret;
	int crit=1;
	unsigned char flag=0;

	printf("------\n");
	printf("Configure detail information for KeyUsage extension.\n");

	crit = ca_ask_yesno("set critical flag",1);
	flag |= (ca_ask_yesno("set digitalSignature flag",1))?(0x80):(0);
	flag |= (ca_ask_yesno("set nonRepudiation flag",1))?(0x40):(0);
	flag |= (ca_ask_yesno("set keyEncipherment flag",1))?(0x20):(0);
	flag |= (ca_ask_yesno("set dataEncipherment flag",1))?(0x10):(0);
	flag |= (ca_ask_yesno("set keyAgreement flag",0))?(0x08):(0);
	flag |= (ca_ask_yesno("set keyCertSign flag",0))?(0x04):(0);
	flag |= (ca_ask_yesno("set cRLSign flag",0))?(0x02):(0);

	if((ret=Extnew_keyusage(flag))==NULL) return NULL;
	ret->critical = crit;
	return ret;
}

CertExt *ca_ask_authkeyid(CA *ca){
	int flag=0;

	printf("------\n");
	printf("Configure detail information for AuthKeyID extension.\n");
	flag |= (ca_ask_yesno("set key identifier",1))?(0x04):(0);
	flag |= (ca_ask_yesno("set issuer DN",0))?(0x02):(0);
	flag |= (ca_ask_yesno("set serial number",0))?(0x01):(0);
	
	return Extnew_authkey_id(ca->cert,flag);
}

CertExt *ca_ask_extcomment(CA *ca,int id,char *txt){
	char buf[256]="";

	printf("------\n");
	printf("Configure detail information for %s extension.\n",txt);
	if(ca_ask_comment("input text",buf,256)) return NULL;
	
	return Extnew_comment(id,buf);
}

CertExt *ca_ask_nstype(CA *ca){
	unsigned char flag=0;

	printf("------\n");
	printf("Configure detail information for NetscapeCertType extension.\n");

	flag |= (ca_ask_yesno("set SSL Client flag",1))?(0x80):(0);
	flag |= (ca_ask_yesno("set SSL Server flag",0))?(0x40):(0);
	flag |= (ca_ask_yesno("set S/MIME flag",1))?(0x20):(0);
	flag |= (ca_ask_yesno("set Object Signing flag",1))?(0x10):(0);
	flag |= (ca_ask_yesno("set SSL CA flag",0))?(0x04):(0);
	flag |= (ca_ask_yesno("set S/MIME CA flag",0))?(0x02):(0);
	flag |= (ca_ask_yesno("set Object Signing CA flag",0))?(0x01):(0);

	return Extnew_ns_flag(flag);
}

/*--------------------------------------------
	set profile extensions
--------------------------------------------*/
int CA_set_extensions(CA *ca,int extoid){
	CertProf *cpf = ca->cprof;
	CertExt  *ext;
	char *txt="";
	int add=0,change=0;

	/* ask add or delete a extension, or cancel */
	if((add = ca_ask_add_or_delete())<0) return 0;

	/* delete current extension if exist */
	Prof_delete_ext(cpf,extoid);

	/* add new extension */
	switch(extoid){
	case OBJ_NS_CERT_CRLURL:
		txt = "Netscape CrlUrl";
		if(add){
			if((ext = ca_ask_extcomment(ca,extoid,txt))==NULL) goto error;
			ext->next = cpf->ext;
			cpf->ext = ext;
		}
		break;
	case OBJ_NS_CERT_COMMENT:
		txt = "Netscape Comment";
		if(add){
			if((ext = ca_ask_extcomment(ca,extoid,txt))==NULL) goto error;
			ext->next = cpf->ext;
			cpf->ext = ext;
		}
		break;
	case OBJ_NS_CERT_TYPE:
		txt = "Netscape CertType";
		if(add){
			if((ext = ca_ask_nstype(ca))==NULL) goto error;
			ext->next = cpf->ext;
			cpf->ext = ext;
		}
		break;
	case OBJ_X509v3_KEY_Usage:
		txt = "KeyUsage";
		if(add){
			if((ext = ca_ask_keyusage(ca))==NULL) goto error;
			ext->next = cpf->ext;
			cpf->ext = ext;
		}
		break;
	case OBJ_X509v3_BASIC:
		txt = "BasicConstraints";
		if(add){
			if((ext = ca_ask_basiccons(ca))==NULL) goto error;
			ext->next = cpf->ext;
			cpf->ext = ext;
		}
		break;
	case OBJ_X509v3_AuthKeyIdt:
		txt = "AuthorityKeyIdentifier";
		if(add){
			if((ext = ca_ask_authkeyid(ca))==NULL) goto error;
			ext->next = cpf->ext;
			cpf->ext = ext;
		}
		break;
	case OBJ_X509v3_SbjKeyIdt:
		txt = "SubjectKeyIdentifier";
		if(add){
			if((ext = Extnew_sbjkey_id(ca->cert))==NULL) goto error;
			ext->next = cpf->ext;
			cpf->ext = ext;
		}
		break;
	}
	if(add) printf("'%s' extension is added to '%s' profile.\n",txt,cpf->name);
	else    printf("'%s' extension is deleted from '%s' profile.\n",txt,cpf->name);
	change++;

    return change;
error:
	return -1;
}

