/* cmptest.c */
/*
 * Copyright (C) 1998-2002
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_io.h"
#include "ok_rand.h"
#include "ok_asn1.h"
#include "ok_tool.h"
#include "ok_cmp.h"

#ifdef __WINDOWS__
# include <malloc.h>
# include <crtdbg.h>

# ifdef   _DEBUG
#  define  SET_CRT_DEBUG_FIELD(a) \
            _CrtSetDbgFlag((a) | _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG))
#  define  CLEAR_CRT_DEBUG_FIELD(a) \
            _CrtSetDbgFlag(~(a) & _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG))
# else
#  define  SET_CRT_DEBUG_FIELD(a)   ((void) 0)
#  define  CLEAR_CRT_DEBUG_FIELD(a) ((void) 0)
# endif
#endif

int test_cmp_header();
int test_cmp_body();
int test_cmp_message();


#ifndef __WINDOWS__
/*
 * CMP test 
 */
int main(int argc,char **argv){
	if(test_cmp_header()) return -1;

	if(test_cmp_body()) return -2;

	if(test_cmp_message()) return -3;

	return 0;
}

#else

int test_cmp_memory();

int main(int argc,char **argv){
	_CrtMemState s1, s2, s3;
	int i;

	_CrtSetReportMode( _CRT_WARN, _CRTDBG_MODE_FILE );
	_CrtSetReportFile( _CRT_WARN, _CRTDBG_FILE_STDOUT );
	_CrtSetReportMode( _CRT_ERROR, _CRTDBG_MODE_FILE );
	_CrtSetReportFile( _CRT_ERROR, _CRTDBG_FILE_STDOUT );
	_CrtSetReportMode( _CRT_ASSERT, _CRTDBG_MODE_FILE );
	_CrtSetReportFile( _CRT_ASSERT, _CRTDBG_FILE_STDOUT );

	//SET_CRT_DEBUG_FIELD( _CRTDBG_DELAY_FREE_MEM_DF);
	SET_CRT_DEBUG_FIELD( _CRTDBG_LEAK_CHECK_DF );
	SET_CRT_DEBUG_FIELD( _CRTDBG_CHECK_ALWAYS_DF );
	_CrtMemDumpAllObjectsSince(&s1);
	_CrtMemCheckpoint( &s1 );

	/* OK_set_errout(stderr); */

	RAND_init();

	/* test CA memory */

	/* test apps memory */

	/* test cmp memory */
	if(i=test_cmp_memory()){
		printf("memory check error -- test_cmp_memory : %d\n",i);
		exit(1);
	}

	RAND_cleanup();

	_CrtMemCheckpoint( &s2 );
	if ( _CrtMemDifference( &s3, &s2, &s1 ) )
		_CrtMemDumpStatistics( &s3 );

	return 0;
}

int test_cmp_memory(){
	_CrtMemState s1, s2, s3;

	/* CMP header check */
	_CrtMemCheckpoint(&s1);
	if(test_cmp_header()) return -1;
	_CrtMemCheckpoint(&s2);

	if ( _CrtMemDifference(&s3,&s1,&s2)){
		_CrtMemDumpStatistics( &s3 ); return -1;}
#if 1
	/* CMP body check */
	_CrtMemCheckpoint(&s1);
	if(test_cmp_body()) return -1;
	_CrtMemCheckpoint(&s2);

	if ( _CrtMemDifference(&s3,&s1,&s2)){
		_CrtMemDumpStatistics( &s3 ); return -1;}
#endif
	/* CMP message check */
	_CrtMemCheckpoint(&s1);
	if(test_cmp_message()) return -1;
	_CrtMemCheckpoint(&s2);

	if ( _CrtMemDifference(&s3,&s1,&s2)){
		_CrtMemDumpStatistics( &s3 ); return -1;}

	return 0;
}

#endif

