/* ecctestfc.c */

#include "aiconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef USE_PTHREAD
#include <pthread.h>
#endif

#include "ok_dsa.h"
#include "ok_asn1.h"

#define TEST_DSAPARAM	"dsapm.der"
#define TEST_DSAPARAM0	"dsapm0.der"
#define TEST_DSAKEY		"dsakey.der"
#define TEST_DSAKEY0	"dsakey0.der"
#define TEST_DSACERT	"pkix-ex1.ber"

int test_dsapm_der(){
	unsigned char *der=NULL,*der2=NULL;
	DSAParam *pm=NULL;
	int i,j,err=-1;

	/* decode and encode DSA Param DER */
	for(j=0;j<11;j++){
		/* read DSA Param DER */
		if((der=ASN1_read_der(TEST_DSAPARAM))==NULL) goto done;

		if((pm=ASN1_read_dsaparam(der,0))==NULL){
			printf("DSA : decode parameter -- error!\n");
			goto done;
		}

		/* write DSA Param DER */
		if((der2=DSAPm_toDER(pm,NULL,&i,0))==NULL){
			printf("DSA : encode parameter -- error!\n");
			goto done;
		}
		if(ASN1_write_der(der2,TEST_DSAPARAM0)) goto done;

		if((j%5)==0) printf("DSA : decode & encode parameter    -- ok : %d\n",j);
	
		/* compare two parameters */
		if(memcmp(der,der2,i)){
			printf("DSA : compare DER parameters der==der2 -- error!\n");
			goto done;
		}
		if((j%5)==0) printf("DSA : compare DER binary der==der2 -- ok : %d\n",j);

		FREE(der2); der2=NULL;
		DSAPm_free(pm); pm=NULL; der=NULL; /* it frees "der" memory */
	}

	err=0;
done:
	if(der2) FREE(der2);
	if((pm==NULL)&&(der)) FREE(der);
	DSAPm_free(pm); /* it frees "der" memory */
	return err;
}

int test_dsakey_der(){
	unsigned char *der=NULL,*der2=NULL;
	Prvkey_DSA *prv=NULL;
	Pubkey_DSA *pub=NULL;
	int i,j,err=-1;

	/* read & write DSA key */
	for(j=0;j<11;j++){
		/* read DSA Privatekey DER */
		if((der=ASN1_read_der(TEST_DSAKEY))==NULL) goto done;

		if((prv=ASN1_read_dsaprv(der))==NULL){
			printf("DSA : decode private key -- error!\n");
			goto done;
		}

		/* write EC Param DER */
		if((der2=DSAprv_toDER(prv,NULL,&i))==NULL){
			printf("DSA : encode private key -- error!\n");
			goto done;
		}
		if(ASN1_write_der(der2,TEST_DSAKEY0)) goto done;

		if((j%5)==0) printf("DSA : decode & encode private key  -- ok : %d\n",j);
	
		/* compare two parameters */
		if(memcmp(der,der2,i)){
			printf("DSA : compare DER parameters der==der2 -- error!\n");
			goto done;
		}
		if((j%5)==0) printf("DSA : compare DER binary der==der2 -- ok : %d\n",j);

		FREE(der2); der2=NULL;
		DSAkey_free((Key*)prv); prv=NULL; der=NULL; /* it frees "der" memory */
	}

	/* compare DSA keys */
	for(j=0;j<11;j++){
		if((pub=DSApubkey_new())==NULL) goto done;
		if((der=ASN1_read_der(TEST_DSAKEY))==NULL) goto done;

		if((prv=ASN1_read_dsaprv(der))==NULL){
			printf("DSA : decode private key -- error!\n");
			goto done;
		}
		DSAprv_2pub(prv,pub);

		if(Key_cmp((Key*)prv,(Key*)prv)){
			printf("DSA : compare private key -- error!\n");
			goto done;
		}
		if(Key_cmp((Key*)pub,(Key*)pub)){
			printf("DSA : compare public key -- error!\n");
			goto done;
		}
		if(!Key_cmp((Key*)prv,(Key*)pub)){
			printf("DSA : compare prv & pub key -- error!\n");
			goto done;
		}
		if(Key_pair_cmp((Key*)prv,(Key*)pub)){
			printf("DSA : compare prv & pub key -- error!\n");
			goto done;
		}

		if((j%5)==0) printf("DSA : compare prv & pub keys -- ok : %d\n",j);

		Key_free((Key*)pub);
		Key_free((Key*)prv); /* it frees "der" memory */
		prv=NULL; der=NULL;
	}
	err=0;
done:
	if(der2) FREE(der2);
	if((prv==NULL)&&(der)) FREE(der);
	DSAkey_free((Key*)prv); /* it frees "der" memory */
	return err;
}

int test_dsa_sig(){
	char *txt = "This is sample text."; /* this should be smaller than 20 byte :-) */
	unsigned char *der=NULL,*sig=NULL;
	Prvkey_DSA *prv=NULL;
	Pubkey_DSA *pub=NULL;
	DSAParam *pm=NULL;
	int i,j,err=-1;

	/* read DSA private key & check signature */
	for(j=0;j<11;j++){
		/* read DSA Privatekey DER */
		if((der=ASN1_read_der(TEST_DSAKEY))==NULL) goto done;

		if((prv=ASN1_read_dsaprv(der))==NULL){
			printf("DSA : decode private key -- error!\n");
			goto done;
		}

		/* get DSA signature */
		if((sig=DSA_get_signature(prv,txt,strlen(txt),&i))==NULL){
			goto done;
		}
		/* verify DSA signature */
		if((pub=DSApubkey_new())==NULL) goto done;
		if(DSAprv_2pub(prv,pub)) goto done;

		i=DSA_vfy_signature(pub,txt,strlen(txt),sig);

		if(i<0){
			printf("cannot verify DSA signature -- error! : %d\n",j);
			goto done;
		}
		if(i>0){
			printf("Bad DSA signature -- verification error! : %d\n",j);
			goto done;
		}

		printf("DSA signature : generate & verify -- ok:%d\n",j);

		DSAkey_free((Key*)prv); prv=NULL;
		DSAkey_free((Key*)pub); pub=NULL;
		FREE(sig); sig=NULL; der=NULL;
	}

	/* generate DSA private key & check signaturte */
	for(j=0;j<11;j++){
		/* read DSA Param DER */
		if((der=ASN1_read_der(TEST_DSAPARAM))==NULL) goto done;

		if((pm=ASN1_read_dsaparam(der,0))==NULL){
			printf("DSA : decode parameter -- error!\n");
			goto done;
		}

		/* generate new private key */
		if((prv=DSAprvkey_new())==NULL) goto done;
		if(DSAprv_generate(pm,prv)) goto done;
		DSAPm_free(pm); pm=NULL;

		/* get DSA signature */
		if((sig=DSA_get_signature(prv,txt,strlen(txt),&i))==NULL) goto done;

		/* verify DSA signature */
		if((pub=DSApubkey_new())==NULL) goto done;
		if(DSAprv_2pub(prv,pub)) goto done;

		i=DSA_vfy_signature(pub,txt,strlen(txt),sig);

		if(i<0){
			printf("cannot verify DSA signature (2) -- error! : %d\n",j);
			goto done;
		}
		if(i>0){
			printf("Bad DSA signature (2) -- verification error! : %d\n",j);
			goto done;
		}

		printf("DSA private key generation -- ok:%d\n",j);

		DSAkey_free((Key*)prv); prv=NULL;
		DSAkey_free((Key*)pub); pub=NULL;
		FREE(sig); sig=NULL; der=NULL;
	}

	err=0;
done:
	if(sig) FREE(sig);
	if((prv==NULL)&&(der)) FREE(der);
	DSAkey_free((Key*)prv);
	DSAkey_free((Key*)pub);
	return err;
}

int test_dsa_gen(){
	char *txt = "This is sample text."; /* this should be smaller than 20 byte :-) */
	unsigned char *sig=NULL;
	Prvkey_DSA *prv=NULL;
	Pubkey_DSA *pub=NULL;
	DSAParam *pm;
	int i=0,j,k,err=-1;
	int bit[]={512,768,1024};

	do{
		printf("----- generating DSA parameter test stage %d (%d bit)-----\n",i,bit[i%3]);

		for(k=0;k<2;k++){
			if((pm=DSAPm_gen_parameter(bit[i%3]))==NULL){
				printf("DSA Parameter generation error occured!!\n");
				goto done;
			}

			/* generate new private key */
			if((prv=DSAprvkey_new())==NULL) goto done;
			if(DSAprv_generate(pm,prv)) goto done;

			/* get DSA signature */
			if((sig=DSA_get_signature(prv,txt,strlen(txt),&j))==NULL) goto done;

			/* verify DSA signature */
			if((pub=DSApubkey_new())==NULL) goto done;
			if(DSAprv_2pub(prv,pub)) goto done;

			j=DSA_vfy_signature(pub,txt,strlen(txt),sig);

			if(j<0){
				printf("cannot verify DSA signature (2) -- error! : %d\n",j);
				goto done;
			}
			if(j>0){
				printf("Bad DSA signature (2) -- verification error! : %d\n",j);
				goto done;
			}
			printf("\nDSA Parameter generation -- signature test ok!!\n");

			DSAPm_free(pm);
			DSAkey_free((Key*)prv);
			DSAkey_free((Key*)pub);
			FREE(sig); sig=NULL;
			pm=NULL; prv=NULL; pub=NULL;
		}

		i++;
	}while(i<3);

	err=0;
done:
	if(sig) FREE(sig);
	DSAPm_free(pm);
	DSAkey_free((Key*)prv);
	DSAkey_free((Key*)pub);
	return err;
}

