/* ecctestfc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_asn1.h"
#include "ok_tool.h"

#include "ok_ecc.h"
#include "ok_ecdsa.h"

#define TEST_ECDSAKEY		"ecpkey.der"
#define TEST_ECDSAKEY0		"#ecpkey.der"

int test_ecdsa_signature(){
	ECParam *E;
	Prvkey_ECDSA *prv;
	Pubkey_ECDSA *pub;
	unsigned char *buf, *dgst;
	char txt[]="This is sample text for generating digest data used elliptic curve signature.\nHmmmm..\n";
	int	i,j,len;

	E=ECPm_get_std_parameter(ECP_X962_prime256v1);

/* 	E=ECPm_gen_parameter(320); */

	for(i=0;i<11;i++){
		prv=ECDSAprvkey_new();
		pub=ECDSApubkey_new();
		
		/* generating ecdsa private key */
		ECDSAprv_generate(E,prv);
		/*LN_print(prv->k);
		  ECp_print(prv->W);*/

		/* generating ecdsa signature */
		ECDSAprv_2pub(prv, pub);
		dgst=OK_do_digest(OBJ_HASH_SHA1,txt,strlen(txt)-i,NULL,&len);
		buf=ECDSA_get_signature(prv, dgst, len, &j);
#if 0
		{
			Prvkey_ECDSA *caprv;
			Cert *ct;
			if((ct=Cert_read_file("ca.cer"))==NULL){
				printf("test x509 cert read file (3) -- error!\n");
				return -1;
			}
			caprv = ASN1_read_ecdsaprv(ASN1_read_der("ecca.key"));

			ct->pubkey = (Key*)pub;
			ct->pubkey_algo = pub->key_type;

			OK_set_cert_sig_algo(OBJ_SIG_SHA1ECDSA);
			FREE(ct->signature);
			ct->signature= NULL;
			if((ct->der=Cert_toDER(ct,(Key*)caprv,NULL,&j))==NULL){
				printf("cannot create DER encoding with cert -- error!\n");
				return -1;
			}
			ASN1_write_der(ct->der,"ecca.cer");
			//ASN1_write_der(prv->der,"ecusr.key");
		}
#endif
		/* verifying ecdsa signature */
		if(ECDSA_vfy_signature(pub,dgst,len,buf)){
			printf("error : ECDSA signature test (%d)\n",i);
			return -1;
		}
		printf("ECDSA signature -- ok:%d\n",i);

		FREE(dgst);
		FREE(buf);
		ECDSAkey_free((Key*)pub);
		ECDSAkey_free((Key*)prv);
	}
	ECPm_free(E);	
	return 0;
}

int test_ecdsakey_der(){
	unsigned char *der=NULL,*der2=NULL;
	Prvkey_ECDSA *prv=NULL;
	Pubkey_ECDSA *pub=NULL;
	int i,j,err=-1;

	/* read & write ECDSA key */
	for(j=0;j<11;j++){
		/* read ECDSA Privatekey DER */
		if((der=ASN1_read_der(TEST_ECDSAKEY))==NULL) goto done;

		if((prv=ASN1_read_ecdsaprv(der))==NULL){
			printf("ECDSA : decode private key -- error!\n");
			goto done;
		}

		/* write EC Param DER */
		if((der2=ECDSAprv_toDER(prv,NULL,&i))==NULL){
			printf("ECDSA : encode private key -- error!\n");
			goto done;
		}
		if(ASN1_write_der(der2,TEST_ECDSAKEY0)) goto done;

		if((j%5)==0) printf("ECDSA : decode & encode private key  -- ok : %d\n",j);

		/* compare two parameters */
		if(memcmp(der,der2,i)){
			printf("ECDSA : compare DER parameters der==der2 -- error!\n");
			goto done;
		}
		if((j%5)==0) printf("ECDSA : compare DER binary der==der2 -- ok : %d\n",j);

		FREE(der2); der2=NULL;
		ECDSAkey_free((Key*)prv); prv=NULL; der=NULL; /* it frees "der" memory */
	}

	/* compare ECDSA keys */
	for(j=0;j<11;j++){
		if((pub=ECDSApubkey_new())==NULL) goto done;
		if((der=ASN1_read_der(TEST_ECDSAKEY))==NULL) goto done;

		if((prv=ASN1_read_ecdsaprv(der))==NULL){
			printf("ECDSA : decode private key -- error!\n");
			goto done;
		}
		ECDSAprv_2pub(prv,pub);

		if(Key_cmp((Key*)prv,(Key*)prv)){
			printf("ECDSA : compare private key -- error!\n");
			goto done;
		}
		if(Key_cmp((Key*)pub,(Key*)pub)){
			printf("ECDSA : compare public key -- error!\n");
			goto done;
		}
		if(!Key_cmp((Key*)prv,(Key*)pub)){
			printf("ECDSA : compare prv & pub key -- error!\n");
			goto done;
		}
		if(Key_pair_cmp((Key*)prv,(Key*)pub)){
			printf("ECDSA : compare prv & pub key -- error!\n");
			goto done;
		}

		if((j%5)==0) printf("ECDSA : compare prv & pub keys -- ok : %d\n",j);

		Key_free((Key*)pub);
		Key_free((Key*)prv); /* it frees "der" memory */
		prv=NULL; der=NULL;
	}

	err=0;
done:
	if(der2) FREE(der2);
	if((prv==NULL)&&(der)) FREE(der);
	ECDSAkey_free((Key*)prv); /* it frees "der" memory */
	return err;
}

