/*
 * large number test functions.
 */
/*
 * Copyright (C) 1998-2002
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "large_num.h"

/* size=1 means 16 bit long */
void make_test_LNm(LNm **a1,LNm **a2,int size){
    unsigned short n1[256];
    unsigned short n2[256];
    int  i;
    
    for(i=0;i<size;i++)
	n1[i]= (unsigned short)rand() * 3;
    for(i=0;i<size-1;i++)
	n2[i]= (unsigned short)rand();

    *a1=LN_alloc_s(size,n1);
    *a2=LN_alloc_s(size-1,n2);
}

int test_plus_minus(void){
	LNm *a1,*a2,*pul,*min;
	unsigned short n1[4];
	ULONG  n2[4];
	int	i,j;

	pul = LN_alloc();
	min = LN_alloc();

	for(j=32;j<129;j+=32){
		for(i=0;i<2001;i++){
			make_test_LNm(&a1,&a2,j);

			LN_plus(a1,a2,pul);
			LN_minus(pul,a2,min);
			if(LN_cmp(a1,min)){
				printf("%d : ERROR eq = %d\n",i,LN_cmp(a1,min));
				return -1;
			}

			LN_minus(a2,a1,min);
			LN_plus(min,a1,pul);
			if(LN_cmp(a2,pul)){
				printf("%d : ERROR -+ = %d\n",i,LN_cmp(a1,min));
				return -1;
			}

			if(!(i%500))printf("+- (%d bit)-- ok:%d\n",j*16,i);
    
			LN_free(a1);
			LN_free(a2);
		}
	}

	/* other test 1 */
	n1[0]=0;
	LN_clean(min);
	a1=LN_alloc_s(1,n1);
	a2=LN_alloc_s(1,n1);
    LN_plus(a1,a2,pul);
    LN_minus(pul,a2,min);
    if(LN_cmp(a1,min)){
		printf("%s : ERROR eq = %d\n","test1",LN_cmp(a1,min));
		return -1;
    }
    LN_free(a1);    LN_free(a2);

	/* other test 2 */
	n2[0]=0xfffffffe;
	a1=LN_alloc_u32(1,n2);
	a2=LN_alloc_u32(1,n2);
    LN_plus(a1,a2,pul);
    LN_minus(pul,a2,min);

    if(LN_cmp(a1,min)){
		printf("%s : ERROR eq = %d\n","test2",LN_cmp(a1,min));
		return -1;
    }
    LN_free(a1);    LN_free(a2);

	/* other test 3 */
	n2[0]=0xffffffff; n2[1]=0xffffffff;
	a1=LN_alloc_u32(2,n2);
	a2=LN_alloc_u32(2,n2);

    LN_plus(a1,a2,pul);
    LN_minus(pul,a2,min);
    if(LN_cmp(a1,min)){
		printf("%s : ERROR eq = %d\n","test3",LN_cmp(a1,min));
		return -1;
    }
    LN_free(a1);    LN_free(a2);

	/* other test 4 */
	n2[0]=0x1000000; n2[1]=0; n2[1]=0; n2[3]=0;
	a1=LN_alloc_u32(4,n2);
	a2=LN_alloc_u32(1,n2);
    LN_minus(a1,a2,min);
    LN_plus(min,a2,pul);
    if(i=LN_cmp(a1,pul)){
		printf("test4 : ERROR eq = %d\n",i);
		return -1;
    }
    LN_free(a1);    LN_free(a2);

	/* other test 5 */
	make_test_LNm(&a1,&a2,64);
	LN_clean(a2);
	LN_plus(a1,a2,pul);
	if(i=LN_cmp(a1,pul)){
		printf("test5 : ERROR eq = %d\n",i);
		return -1;
	}
    LN_free(a1);    LN_free(a2);

	LN_free(pul);
	LN_free(min);

	printf("plus minus test .. ok\n");
	return 0;
}

int test_mul_div_mod(void){
	LNm *a1,*a2;
    LNm *b,*c,*d,*e;
	ULONG  n2[16];
	ULONG  a1n[]={
		0x5500f4cc,0xcabee396,0xdf404457,0x53131aea,0xc5b4ba67,0x5ac54b59,
		0x148f222e,0x30e76464,0xc38e2503,0xfec61789,0x92a35d64};
	ULONG  a2n[]={
		0x0001394e,0x6a38470a,0xb92f5a44,0x2c744dad,
		0x89b8c531,0x67076fff};
	int	i,j;

	b = LN_alloc();
	c = LN_alloc();
	d = LN_alloc();
	e = LN_alloc();

	/* div mod test 1 */
	a1=LN_alloc_u32(11,a1n);
	a2=LN_alloc_u32(6,a2n);

	for(i=0;i<1001;i++){
		LN_div_mod(a1,a2,b,c);
		LN_multi_std(b,a2,d);
		LN_plus(d,c,e);

		if(LN_cmp(a1,e)){
			printf("ERROR! i=%d::%d\n",i,LN_cmp(a1,e));
			return -1;}

		if(!(i%200)) printf("/* -- ok:%d\n",i);
	}
	LN_free(a1);LN_free(a2);

    /* div mod test */
	for(j=32;j<129;j+=32){
		for(i=0;i<2001;i++){
			make_test_LNm(&a1,&a2,j);

			LN_div_mod(a1,a2,b,c);
			LN_multi_std(b,a2,d);

			LN_plus(d,c,e);

			if(LN_cmp(a1,e)){
				printf("a1/a2 = b...c,\na2*c = d, d+c = e\n");
				LN_print(a1); LN_print(a2);

				LN_print(b); LN_print(c);
				LN_print(d); LN_print(e);

				printf("ERROR! i=%d::%d\n",i,LN_cmp(a1,e));
				return -1;
			}
		
			if(!(i%500)) printf("/* (%d bit) -- ok:%d\n",j*16,i);

			LN_free(a1);
			LN_free(a2);
		}
	}

    /* div mod test 2 */
	for(j=128;j<129;j+=32){
		for(i=0;i<1001;i++){
			make_test_LNm(&a1,&a2,j);

			LN_multi(a1,a1,b);
			LN_div_mod(b,a1,c,d);

			if(LN_cmp(a1,c)&&d->top){
				printf("a1*a2 = b,\nb/a2 = c...d\n");
				LN_print(a1); LN_print(a2);

				LN_print(b); LN_print(c);
				LN_print(d);

				printf("ERROR! i=%d::%d\n",i,LN_cmp(a1,c));
				return -1;
			}
		
			if(!(i%200)) printf("*/ (%d bit)-- ok:%d\n",j*16,i);
			LN_free(a1);
			LN_free(a2);
		}
	}

    /* sqr test */
	for(j=32;j<129;j+=32){
		for(i=0;i<1001;i++){
			make_test_LNm(&a1,&a2,j);

			LN_multi(a1,a1,b);
			LN_sqr(a1,c);

			if(LN_cmp(b,c)){
				printf("a1*a1 = b,\na1^2 = c\n");
				LN_print(a1); LN_print(b); LN_print(c);

				printf("ERROR! i=%d::%d\n",i,LN_cmp(b,c));
				return -1;
			}
		
			LN_free(a1);	LN_free(a2);
			if(!(i%200)) printf("x^2 (%d bit)-- ok:%d\n",j*16,i);
		}
	}

	/* other test 1 */
	n2[0]=0xffffffff; n2[1]=0xffffffff;
	n2[2]=0xffffffff; n2[3]=0xffffffff;
	a1=LN_alloc_u32(4,n2);
	a2=LN_alloc_u32(4,n2);

	n2[0]=0xffffffff; n2[1]=0xffffffff; n2[2]=0xffffffff; n2[3]=0xfffffffe; 
	n2[4]=0x00000000; n2[5]=0x00000000; n2[6]=0x00000000; n2[7]=0x00000001; 
	LN_set_num(c,8,n2);
	
    LN_multi(a1,a2,b);
	LN_print(b);

    if(LN_cmp(b,c)){
		LN_print(c);
		printf("%s : ERROR eq = %d\n","multi test 1",LN_cmp(b,c));
		return -1;
    }
    LN_free(a1);    LN_free(a2);

	/* other test 2 */
	n2[0]=0xffffffff; n2[1]=0xffffffff; n2[2]=0xffffffff; n2[3]=0xffffffff;
	a1=LN_alloc_u32(4,n2);
	n2[0]=0x00000001; n2[1]=0xffffffff; n2[2]=0x33333333; n2[3]=0xcccccccc;
	a2=LN_alloc_u32(3,n2);
	LN_div_mod(a1,a2,b,c);
    LN_free(a1);    LN_free(a2);

	/* other test 3 */
	n2[0]=0xffffffff; n2[1]=0xffffffff; n2[2]=0xffffffff; n2[3]=0xffffffff;
	a1=LN_alloc_u32(4,n2);
	LN_long_multi(a1,1,b);
    if(LN_cmp(a1,b)){
		LN_print(b);
		printf("%s : ERROR eq = %d\n","multi test 3",LN_cmp(a1,b));
		return -1;
    }
    LN_free(a1);

	/* other test 4 */
	n2[0]=0xffffffff; n2[1]=0xffffffff; n2[2]=0xffffffff; n2[3]=0xffffffff;
	a1=LN_alloc_u32(4,n2);
	a2=LN_alloc_u32(1,n2);
	LN_long_multi(a1,0xffffffff,b);
	LN_multi(a1,a2,c);
    if(LN_cmp(b,c)){
		LN_print(b);
		LN_print(c);
		printf("%s : ERROR eq = %d\n","multi test 4",LN_cmp(b,c));
		return -1;
    }
    LN_free(a1);    LN_free(a2);

	/* other test 5 */
	n2[0]=0x0000ffff; n2[1]=0xffffffff;
	a1=LN_alloc_u32(2,n2);
	LN_multi(a1,a1,b);
	LN_sqr(a1,c);
    if(LN_cmp(b,c)){
		LN_print(b);
		LN_print(c);
		printf("%s : ERROR eq = %d\n","multi test 5 (sqr)",LN_cmp(b,c));
		return -1;
    }
    LN_free(a1);

	/* other test 6 */
	n2[0]=0xffffffff; n2[1]=0xffffffff;
	a1=LN_alloc_u32(2,n2);
	LN_multi(a1,a1,b);
	LN_sqr(a1,c);
    if(LN_cmp(b,c)){
		LN_print(b);
		LN_print(c);
		printf("%s : ERROR eq = %d\n","multi test 6 (sqr)",LN_cmp(b,c));
		return -1;
    }
    LN_free(a1);


    LN_free(b); LN_free(c); LN_free(d); LN_free(e);

    printf("div and mod test .. ok\n");
	return 0;
}

int test_karatsuba(void){
	LNm *a1,*a2;
    LNm *b,*c,*d,*e;
	int	i,j;

	/* test1 karatsuba */
	ULONG tst0[]={
		0x00ab35ff,0xd8ba5fa8,0xe6d3a243,0x846a9730,
		0x33818f75,0x2dee6242,0x70d8c828};

	ULONG tst1[]={
		0x00ab35ff,0xd8ba5fa8,0xe6d3a243,0x846a9730,0x33818f75,
		0x2dee6242,0x70d8c828,0x46bdcd0c,0xbb868c61,0x4c58de05,
		0xd7ffd46e,0x671f165b,0x5d016613,0x571df358,0x5584d059,
		0x0b4b8100,0x00000000,0x00000000,0x00000000,0x00000000,
		0x00000000,0x00000000};

	ULONG tst2[]={
		0xe7a01eb2,0xe2c10f1a,0xfbdd9bb0,0x89a8a35c,0xd9e3847d,
		0x718f1100,0x00000000,0x00000000,0x00000000,0x00000000,
		0x00000000,0x00000000,0x00000000};

	b = LN_alloc();
	c = LN_alloc();
	d = LN_alloc();
	e = LN_alloc();

	/* karatsuba test1 */
	LN_set_num(b,7,tst0);
    LN_multi_kara(b,b,c);	LN_multi_std(b,b,d);

	if(LN_cmp(c,d)){ LN_print(b); LN_print(c); LN_print(d);
		printf("Karatsuba test1: ERROR eq = %d\n",LN_cmp(c,d));
		return -1;
    }else
		printf("Karatsuba test1 -- ok\n",LN_cmp(c,d));

	/* karatsuba test2 */
	LN_set_num(b,22,tst1);
    LN_multi_kara(b,b,c);	LN_multi_std(b,b,d);

	if(LN_cmp(c,d)){ LN_print(b); LN_print(c); LN_print(d);
		printf("Karatsuba test2: ERROR eq = %d\n",LN_cmp(c,d));
		return -1;
    }else
		printf("Karatsuba test2 -- ok\n",LN_cmp(c,d));

	/* karatsuba test3 */
	LN_set_num(b,7,tst2);
    LN_multi_kara(b,b,c);	LN_multi_std(b,b,d);

	if(LN_cmp(c,d)){ LN_print(b); LN_print(c); LN_print(d);
		printf("Karatsuba test3: ERROR eq = %d\n",LN_cmp(c,d));
		return -1;
    }else
		printf("Karatsuba test3 -- ok\n",LN_cmp(c,d));

	/* karatsuba test 4 */
	for(j=32;j<129;j+=32){
		for(i=0;i<1001;i++){
			make_test_LNm(&a1,&a2,j);

			LN_multi_kara(a1,a2,b);
			LN_multi_std(a1,a2,c);

		    if(LN_cmp(b,c)){
				LN_print(a1);	LN_print(b);	LN_print(c);
				printf("%s (%d): ERROR eq = %d\n","kara-multi test",i,LN_cmp(b,c));
				return -1;
			}

		    LN_multi_kara(a2,a2,b);
			LN_multi_std(a2,a2,c);
		    if(LN_cmp(b,c)){
				printf("%s (%d): ERROR eq = %d\n","kara-multi test",i,LN_cmp(b,c));
				return -1;
		    }

		    LN_multi_kara(a2,a1,b);
			LN_multi_std(a2,a1,c);
		    if(LN_cmp(b,c)){
				printf("%s (%d): ERROR eq = %d\n","kara-multi test",i,LN_cmp(b,c));
				return -1;
		    }

			if(!(i%200)) printf("karatsuba multi (%d bit) -- ok:%d\n",j*16,i);
		    LN_free(a1);    LN_free(a2);
		}
	}

	/* karatsuba test 5 */
	for(j=32;j<129;j+=32){
		for(i=0;i<1001;i++){
			make_test_LNm(&a1,&a2,j);

		    LN_sqr_kara(a1,b);
#ifdef __WINDOWS__
			LN_sqr(a1,c);
#else
			LN_sqr_std(a1,c);
#endif
		    if(LN_cmp(b,c)){
				LN_print(a1);	LN_print(b);	LN_print(c);
				printf("%s (a1:%d): ERROR eq = %d\n","kara-sqr test",i,LN_cmp(b,c));
				return -1;
		    }

		    LN_sqr_kara(a2,b);
#ifdef __WINDOWS__
			LN_sqr(a2,c);
#else
			LN_sqr_std(a2,c);
#endif
		    if(LN_cmp(b,c)){
				printf("%s (a2:%d): ERROR eq = %d\n","kara-sqr test",i,LN_cmp(b,c));
				return -1;
		    }

			if(!(i%200)) printf("karatsuba sqr (%d bit) -- ok:%d\n",j*16,i);
		    LN_free(a1);    LN_free(a2);
		}
	}

    LN_free(b); LN_free(c); LN_free(d); LN_free(e);
    printf("karatsuba mul and sqr test .. ok\n");
	return 0;
}

int test_shift(void){
	LNm *a1,*a2;
    LNm *b,*c;
	ULONG  n2[16];
	int	i,j,k;

	b = LN_alloc();
	c = LN_alloc();

	for(k=32;k<129;k+=32){
		for(i=0;i<501;i++){
			make_test_LNm(&a1,&a2,k);

			for(j=0;j<=31;j++){
				LN_lshift32(a1,j,b);
				LN_rshift32(b,j,c);
			    if(LN_cmp(a1,c)){
					LN_print(a1);
					LN_print(c);
					printf("shift test (%d,%d): ERROR eq = %d\n",i,j,LN_cmp(a1,c));
					return -1;
				}
			}

			if(!(i%100)) printf("shift (%d bit) -- ok:%d\n",k*16,i);
			LN_free(a1); LN_free(a2);
		}	
	}

	/* other test 1 */
	n2[0]=0x0fffffff; n2[1]=0xfffffff0;
	a1=LN_alloc_u32(1,n2);
	a2=LN_alloc_u32(1,&n2[1]);
	LN_lshift32(a1,4,b);
    if(LN_cmp(b,a2)){
		LN_print(b);
		LN_print(a2);
		printf("%s : ERROR eq = %d\n","shift test 1",LN_cmp(b,a2));
		return -1;
    }
    LN_free(a1); LN_free(a2);

	/* other test 2 */
	n2[0]=0x0fffffff; n2[1]=0x00ffffff;
	a1=LN_alloc_u32(1,n2);
	a2=LN_alloc_u32(1,&n2[1]);
	LN_rshift32(a1,4,b);
    if(LN_cmp(b,a2)){
		LN_print(b);
		LN_print(a2);
		printf("%s : ERROR eq = %d\n","shift test 2",LN_cmp(b,a2));
		return -1;
    }
    LN_free(a1); LN_free(a2);

    LN_free(b); LN_free(c);
    printf("left and right shift test .. ok\n");
	return 0;
}

int test_long(void){
	LNm *a1,*a2;
    LNm *b,*c,*d;
	int	i,j;

	b = LN_alloc();
	c = LN_alloc();
	d = LN_alloc();

	for(j=32;j<129;j+=32){
		for(i=0;i<2001;i++){
			ULONG ul;
			make_test_LNm(&a1,&a2,j);

			ul = a2->num[LN_MAX-1];
			LN_copy(a1,b);
			LN_long_add(a1,ul);
			LN_long_sub(a1,ul);

		    if(LN_cmp(a1,b)){
				LN_print(a1);
				LN_print(b);
				printf("%s : ERROR eq = %d\n","long +-",LN_cmp(a1,b));
				return -1;
			}

			LN_long_multi(a1,ul,b);
			LN_long_div(b,ul,c);

		    if(LN_cmp(a1,c)){
				LN_print(a1);
				LN_print(c);
				printf("%s : ERROR eq = %d\n","long */",LN_cmp(a1,c));
				return -1;
			}

			if(!(i%500)) printf("long calc (%d bit) -- ok:%d\n",j*16,i);
			LN_free(a1); LN_free(a2);
		}	
	}

    LN_free(b); LN_free(c); LN_free(d);
    printf("long calc test .. ok\n");
	return 0;
}

int test_ext_euclid(void){
	unsigned short p1[]={	/* p-1 */
	0xcba9,0xe6ba,0xcf43,0x06f1,0xfe88,0xfaa3,0x01fb,0x08d0,
	0xd4ed,0xb47f,0xef57,0xf4d4,0xc2fc,0x045f,0x96af,0xb59d,
	0xa247,0x9769,0xcbae,0xbecb,0xf90c,0xf466,0x05f5,0xfa71,
	0xf931,0xd509,0x0e6f,0xa138,0xf300,0xc800,0xa672,0xe95e};
	unsigned short p2[]={ /* q-1 */
	0xc7f5,0x4c4f,0x4ced,0x39a6,0x54b7,0x3d40,0x74ee,0x6c85,
	0x68a0,0x6ff8,0x76cd,0x42ea,0x227c,0x5d54,0x62d1,0x30f9,
	0x725c,0x04ac,0x239a,0x1044,0x0566,0x39ab,0x2cc8,0x6709,
	0x6263,0x73f2,0x686e,0x672e,0x4306,0x7c6c,0x0f35,0x60fc};
	unsigned short inv[]={
	0x202d,0xcdfc,0xaa6b,0xe222,0x05a8,0x1439,0xe1a5,0xad5e,
	0xd71a,0xb9fb,0xdf9a,0xfa7e,0x1afd,0x164b,0x9a73,0x63e1,
	0x88fa,0xff75,0x093a,0xe5ba,0x4db1,0xb409,0xc724,0x1ff7,
	0xc5f5,0x066f,0xdd8d,0x64f6,0xeb10,0x6c93,0x9a51,0x93cc,
	0x1d64,0x96a5,0x7336,0x75f1,0x1606,0x8a1c,0xafb9,0x6709,
	0xd47d,0x5814,0xb0c5,0x09be,0x31d7,0xb1ae,0xdad5,0x4983,
	0xeb92,0xe7d2,0x57ac,0x64ea,0x06c9,0x8a37,0xe8cd,0x70b9,
	0x60bf,0x55b3,0xf5ff,0xb57c,0xdfc0,0x6808,0x85c3,0x3ac9};
	unsigned short ns[]={
	0x9f14,0x38aa,0x8f06,0x4960,0xd69e,0x9374,0x61db,0x7bed,
	0x01f1,0x26e2,0x3c08,0x9d49,0x3c23,0x03f8,0xd4b4,0xa327,
	0x4665,0xb2a4,0x8024,0xbf46,0x6034,0x8ee3,0x58ba,0xe8b0,
	0xb0ab,0x195f,0x385a,0x6c07,0x7e38,0x6b22,0xae02,0xc870,
	0x0631,0x65ab,0xb9dc,0xc2df,0x93b2,0xfdfe,0xfe20,0xeb7f,
	0x269f,0xdd20,0x3505,0x836e,0x9f45,0x736e,0x05ae,0x1763,
	0x9b05,0x91af,0x0075,0x579e,0xbf0c,0xdc66,0x677d,0xf41e,
	0xd91a,0x5e1b,0x2516,0xd59f,0xe560,0x4c2d,0xc316,0xf888};
	LNm *p,*b,*n,*q,*r;
	int i,j;

	p=LN_alloc();
	b=LN_alloc(); n=LN_alloc();
	q=LN_alloc(); r=LN_alloc();
	LN_long_set(b,0x10001);

	/* test ext_euclid -- 1 */
	/* set phai */
	LN_set_num_s(p,32,p1);
	LN_set_num_s(q,32,p2);
	LN_multi(p,q,n);	/* n=(p-1)(q-1) */

	LN_set_num_s(q,64,ns);
	if(LN_cmp(n,q)){ /* should be o.k. */
		printf("error: test_ext_euclid : multi(p,q,n)\n");
		return -1;
	}

	LN_mod_inverse(b,n,r);	/* r= b^-1 mod n */
	LN_set_num_s(q,64,inv);
	if(LN_cmp(r,q)){
		printf("error: test_ext_euclid 1\n");
		return -1;
	}
	printf("ext_euclid test 1 .. ok\n");

	/* test ext_euclid -- 2 */
	LN_init_lexp_tv();
	for(j=32;j<129;j+=32){
		for(i=0;i<101;i++){
			/* any n is ok */
			LN_set_rand(n,j*2,(unsigned short)rand());
			LN_mod_inverse(b,n,r);	/* r= b^-1 mod n */

			LN_mul_mod(b,r,n,p);
			if((p->top!=1)||(p->num[LN_MAX-1]!=1)){
				printf("error: test_ext_euclid 2\n");
				LN_print(p);
				return -1;
			}
			
			if(!(i%20))
				printf("ext_euclid test 2 (%d bit) .. ok:%d\n",j*16,i);
		}
	}

	LN_free(p);
	LN_free(b); LN_free(n); LN_free(q); LN_free(r);
	return 0;
}

int test_exp_mod(void){
	unsigned short p1[]={	/* p */
	0xcba9,0xe6ba,0xcf43,0x06f1,0xfe88,0xfaa3,0x01fb,0x08d0,
	0xd4ed,0xb47f,0xef57,0xf4d4,0xc2fc,0x045f,0x96af,0xb59d,
	0xa247,0x9769,0xcbae,0xbecb,0xf90c,0xf466,0x05f5,0xfa71,
	0xf931,0xd509,0x0e6f,0xa138,0xf300,0xc800,0xa672,0xe95f};

	unsigned short p2[]={ /* q-1 */
	0xc7f5,0x4c4f,0x4ced,0x39a6,0x54b7,0x3d40,0x74ee,0x6c85,
	0x68a0,0x6ff8,0x76cd,0x42ea,0x227c,0x5d54,0x62d1,0x30f9,
	0x725c,0x04ac,0x239a,0x1044,0x0566,0x39ab,0x2cc8,0x6709,
	0x6263,0x73f2,0x686e,0x672e,0x4306,0x7c6c,0x0f35,0x60fd};

    LNm *a1,*a2;
    LNm *b,*c,*d,*e;
	LNmt_ctx *ctx;
	int i,j;

	b=LN_alloc();
	c=LN_alloc();
	d=LN_alloc();
	e=LN_alloc();

	/* exp_mod test -- 1 */
	LN_init_lexp_tv();
	for(j=32;j<129;j+=32){
		for(i=0;i<11;i++){
			/* any n is ok */
		    make_test_LNm(&a1,&a2,j);
			LN_set_rand(b,j*2,(unsigned short)rand());

			LN_exp_mod(b,a2,a1,c);

			LN_long_sub(a2,1);
			LN_exp_mod(b,a2,a1,d);
			LN_mul_mod(d,b,a1,e);
			
			if(LN_cmp(c,e)){
				printf("error: test_exp_mod 1\n");
				LN_print(e);
				return -1;
			}
			if(!(i%2))
				printf("exp_mod test 1 (%d bit) .. ok:%d\n",j*16,i);
			LN_free(a1); LN_free(a2);
		}
	}

	/* exp_mod test -- 2 */
    make_test_LNm(&a1,&a2,64);

	LN_set_num_s(b,32,p1); /* set prime */
	p1[31]&=0xfffe;
	LN_set_num_s(c,32,p1); /* set prime-1 */
	LN_set_num_s(a1,4,p2);
	LN_exp_mod(a1,c,b,e);  /* a1^(p-1) mod p = 1 */
	if((e->top!=1)||(e->num[LN_MAX-1]!=1)){
		printf("error: test_exp_mod 2\n");
		LN_print(e);
		return -1;
	}
	printf("exp_mod test 2 .. ok\n");

	/* test montgomery exp_mod -- 1 */
	/* get montgomery context */
	ctx=LNmt_get_ctx(b);

	LNmt_exp_mod(ctx,a1,c,b,e);  /* a1^(p-1) mod p = 1 */
	if((e->top!=1)||(e->num[LN_MAX-1]!=1)){
		printf("error: montgomery exp_mod 1\n");
		LN_print(e);
		return -1;
	}
	printf("montgomery exp_mod test 1 .. ok\n");

	/* test montgomery exp_mod -- 2 */
	/* exp_mod test */
	printf("standard   : ");
	for(i=0;i<32;i++){
		LN_exp_mod(a1,c,b,e);
		if((e->top==1)&&(e->num[LN_MAX-1]==1)){
			printf("+"); fflush(stdout);
		}else{
			printf("error: test_exp_mod 1\n");
			LN_print(e);
			return -1;
		}
		LN_long_add(a1,1);
	}

	printf("\nmontgomery : ");
	for(i=0;i<32;i++){
		LNmt_exp_mod(ctx,a1,c,b,e);
		if((e->top==1)&&(e->num[LN_MAX-1]==1)){
			printf("+"); fflush(stdout);
		}else{
			printf("error: test_montgomery_exp_mod 1\n");
			LN_print(e);
			return -1;
		}
		LN_long_add(a1,1);
	}
	printf("\nexp_mod test 2 .. ok\n");

	LN_free(b); LN_free(c); LN_free(d); LN_free(e);
	LN_free(a1); LN_free(a2);
	LNmt_ctx_free(ctx);
	return 0;
}


int test_rsa(void){
    unsigned short sn[]={
	0xed3e,0x206e,0x179f,0x7c27,0x6d0d,0x834c,0xf64b,0xe264,
	0x4b9d,0x6f0d,0xbbbd,0x2956,0xdd70,0x3f64,0x7f40,0x660b,
	0xbe87,0xcb3c,0x656a,0xd501,0xf647,0x1759,0xe62f,0xece6,
	0x3735,0xadfa,0x037f,0x1c8c,0xee5e,0x00eb,0x9580,0x1fab
    };
    unsigned short se[]={
	0x0001,0x0001
    };
    unsigned short sd[]={
	0x1726,0x57cd,0xc65a,0x56a8,0x1639,0x1a55,0xd936,0xc069,
	0x9f03,0x46c1,0xe54e,0xe908,0xc3e4,0xdf1a,0xb45c,0x1958,
	0xddea,0x70f7,0xa236,0x9533,0xe67f,0x4aff,0xc4b3,0xa724,
	0x1dd2,0x1572,0x1e9d,0x01ad,0x0ec0,0x58cb,0x368d,0xe311
    };
	LNm *n,*e,*d;
    LNm *a,*b,*c;
    unsigned char  in[130],cry[130],out[130];

    n=LN_alloc_s(32,sn);
    e=LN_alloc_s(2,se);
    d=LN_alloc_s(32,sd);

    strcpy(in,"This is test.");
    memset(cry,0,130);
    memset(out,0,130);

	a = LN_alloc_c(strlen(in),in);
	b = LN_alloc();
	c = LN_alloc();

	LN_exp_mod(a,e,n,b);
	LN_exp_mod(b,d,n,c);

	if(!LN_cmp(a,c))
		printf("rsa (exp_mod) test .. ok\n");
	else{
		printf("n,e,d\n");
		LN_print(n); LN_print(e); LN_print(d);
		printf("a,b,c\n");
		LN_print(a); LN_print(b); LN_print(c);
		printf("error: a != c ... rsa (exp_mod) test\n");
		return -1;
	}
//	LN_get_num_c(out,len,to);
	LN_free(a);	LN_free(b); LN_free(c);
	LN_free(d);	LN_free(e); LN_free(n);
	return 0;
}

int test_prime(void){
	LNm *a1;
	int i;

	for(i=32;i<129;i+=32){
		a1=LN_alloc();
		LN_prime(i,a1,1);
		LN_print(a1);
		LN_free(a1);
	}
	printf("test prime .. end\n");
	return 0;
}

int test_sqrt(void){
	LNm *a1,*a2;
    LNm *b,*c,*p;
	int	i,j;

	b = LN_alloc();
	c = LN_alloc();
	p = LN_alloc();

	for(j=16;j<65;j+=16){
		for(i=0;i<101;i++){
			make_test_LNm(&a1,&a2,j);

			LN_sqrt(a1, b);
			LN_sqr(b,c);
			LN_long_add(b,1);
			LN_sqr(b,p);

			if((LN_zcmp(c,a1)>0)||(LN_zcmp(a1,p)>0)){
				printf("error: test sqrt (%d)\n",i);
				LN_print(a1);
				LN_print(b);
				LN_print(c);
				LN_print(p);
				return -1;
			}

			if(!(i%10)) printf("test sqrt (%d bit) -- ok:%d\n",j*16,i);

			LN_free(a1); LN_free(a2);
		}
	}

	LN_init_lexp_tv();
/*	for(i=0;i<101;i++){
		int j;
		make_test_LNm2(&a1,&a2);
		a1->num[LN_MAX-1] |= 0x1;

		LN_prime(32,p,0);

		LN_sqr_mod(a2,p,b);
		if((j=LN_jacobi(b,p))==-1){
			printf("error: test jacobi (%d) = %d\n",i,j);
			LN_print(b);
			LN_print(a1);
			exit(0);
		}

		if(!(i%10)) printf("test jacobi -- ok:%d\n",i);

		LN_free(a1); LN_free(a2);
	}*/

	for(j=16;j<65;j+=16){
		for(i=0;i<11;i++){
			LN_prime(j*2,p,0);
			make_test_LNm(&a1,&a2,j);

			LN_sqr_mod(a2,p,b);
			if(LN_mod_sqrt(b,p,c)){
				LN_free(a1); LN_free(a2);
				continue;}

			LN_minus(p,c,b);
			if(LN_cmp(a2,b)&&LN_cmp(a2,c)){
				printf("error: test squre root\n");
				LN_print(b);
				LN_print(c);
				LN_print(a2);
				return -1;
			}

			if(!(i%2)) printf("test mod_sqrt (%d bit) -- ok:%d\n",j*16,i);

			LN_free(a1); LN_free(a2);
		}
	}

	LN_free(b); LN_free(c); LN_free(p);
	return 0;
}

