/*
 * large number test functions.
 */
/*
 * Copyright (C) 1998-2002
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *	this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *	this list of conditions and the following disclaimer in the documentation
 *	and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *	display the following acknowledgment:
 *	"This product includes software developed by Akira Iwata Laboratory,
 *	Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *	acknowledgment:
 *	"This product includes software developed by Akira Iwata Laboratory,
 *	 Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "large_num.h"


/*=========================================================
  large number calculation test
=========================================================*/
int main(argc,argv)
int  argc;
char **argv;
{
	LNm *b,*c,*d,*e;
	int i;
	ULONG ts;
	unsigned char tc[]={0x01,0x02,0x03,0x04,0x05};
	unsigned char td[16];

	b = LN_alloc();
	c = LN_alloc();
	d = LN_alloc();
	e = LN_alloc();

	/* set and print test */
	LN_set_num_c(b,1,tc);
	LN_print(b);

	LN_set_num_c(b,2,tc);
	LN_print(b);

	LN_set_num_c(b,3,tc);
	LN_print(b);

	LN_set_num_c(b,4,tc);
	LN_print(b);

	LN_set_num_c(b,5,tc);
	LN_print(b);

	LN_get_num_c(b,5,td);
	for(i=0;i<5;i++) printf("%.2x ",td[i]);
	printf("\n");

	LN_get_num_c(b,4,td);
	for(i=0;i<4;i++) printf("%.2x ",td[i]);
	printf("\n");

	LN_get_num_c(b,3,td);
	for(i=0;i<3;i++) printf("%.2x ",td[i]);
	printf("\n");

	LN_get_num_c(b,2,td);
	for(i=0;i<2;i++) printf("%.2x ",td[i]);
	printf("\n");

	LN_get_num_c(b,1,td);
	for(i=0;i<1;i++) printf("%.2x ",td[i]);
	printf("\n");

	/* long add test */
	LN_long_add(b,0xffffffff);
	LN_print(b);

	/* long mod test */
	LN_set_num_c(b,3,tc);
	LN_long_mod(b,3,&ts);
	printf("mod = %d :: ",ts); LN_print(b);

	printf(" -- print and set test end -- \n");

	if(test_plus_minus())
		exit(1);

	if(test_mul_div_mod())
		exit(1);

	if(test_karatsuba())
		exit(1);

	if(test_shift())
		exit(1);

	if(test_long())
		exit(1);

	if(test_ext_euclid())
		exit(1);

	if(test_exp_mod())
		exit(1);

	if(test_rsa())
		exit(1);

	if(test_prime())
		exit(1);

	if(test_sqrt())
		exit(1);

	LN_free(b);
	LN_free(c);
	LN_free(d);
	LN_free(e);

	return 0;
}


