/* pemtestfc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_asn1.h"
#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_base64.h"
#include "ok_pem.h"
#include "ok_tool.h"

int test_base64(){
	unsigned char in[512],*dec;
	char *enc;
	int i,size;

	for(i=0;i<512;i++) in[i] = i&0xff;

	for(i=0;i<21;i++){
		enc=Base64_encode(512,in,16);
		dec=Base64_decode(enc,&size);

		if((memcmp(in,dec,512))||(size!=512)){
			printf("test base 64 encode & decode -- error!\n");
			return -1;
		}

		if((i%5)==0)printf("test base 64 encode & decode -- ok : %d\n",i);
		FREE(enc); FREE(dec);
		in[i]=i+3;
	}

	return 0;
}

int test_pem_cry(){
	unsigned char *ivc="abcdefgh";
	unsigned char in[512],*cry,*out;
	int i,len;

	for(i=0;i<512;i++) in[i] = i&0xff;

	OK_set_passwd("abcde");
	for(i=0;i<21;i++){
		len=512;
		cry=PEM_msg_encrypt(in,&len,ivc,OBJ_CRYALGO_DESCBC);
		out=PEM_msg_decrypt(cry,len,ivc,OBJ_CRYALGO_DESCBC);
		if(memcmp(in,out,512)){
			printf("test msg encrypt & decrypt 1 -- error!\n");
			return -1;
		}
		FREE(cry); FREE(out);

		len=512;
		cry=PEM_msg_encrypt(in,&len,ivc,OBJ_CRYALGO_3DESCBC);
		out=PEM_msg_decrypt(cry,len,ivc,OBJ_CRYALGO_3DESCBC);
		if(memcmp(in,out,512)){
			printf("test msg encrypt & decrypt 2 -- error!\n");
			return -1;
		}
		FREE(cry); FREE(out);

		if((i%5)==0)printf("test msg encrypt & decrypt -- ok : %d\n",i);
		in[i]=i+3;
	}
	OK_clear_passwd();
	return 0;
}

int test_pem_msg(){
	char *in,*out;
	int i,j,len;

	in="This is pem test letter\n---!#$%&'&'()---\nEnd.";

	OK_set_passwd("abcde");
	for(i=0;i<11;i++){
		j=strlen(in)+1;
		if(PEM_write_message(in,j,"tmp.txt")){
			printf("test pem msg file -- error in PEM_write_message\n");
			return -1;
		}
		out=PEM_read_message("tmp.txt",&len);
		/* return value "len" is not as same size as original "in" size.
		 * because message had been encrypted with specific size of blocks (like 64bit long).
		 */
		if(strcmp(in,out)){
			printf("test pem msg file -- error!\n");
			return -1;
		}
		FREE(out);
		if((i%5)==0)printf("test pem msg file -- ok : %d\n",i);
	}
	OK_clear_passwd();
	return 0;
}

int test_pem_pki(){
	Cert	*cert;
	Cert	*req;
	CRL  	*crl;
	CertPair	*ccp;
	Prvkey_RSA	*rsa;
	Prvkey_DSA	*dsa;
	Prvkey_ECDSA *ecdsa;
	DSAParam *dpm;
	ECParam	 *E;

	/* certificate */
	if((cert=PEM_read_cert("test.pem"))==NULL){
		printf("test pem read cert file -- error!\n");
		return -1;
	}
	if(PEM_write_cert(cert,"#test.cer")){
		printf("test pem write cert file -- error!\n");
		return -1;
	}
	Cert_free(cert);
	printf("test pem read & write cert file -- ok\n");

	/* CRL */
	if((crl=PEM_read_crl("test.pem"))==NULL){
		printf("test pem read CRL file -- error!\n");
		return -1;
	}
	if(PEM_write_crl(crl,"#test.crl")){
		printf("test pem write CRL file -- error!\n");
		return -1;
	}
	CRL_free(crl);
	printf("test pem read & write CRL file -- ok\n");

	/* certificate request */
	if((req=PEM_read_req("test.pem"))==NULL){
		printf("test pem read cert req file -- error!\n");
		return -1;
	}
	if(PEM_write_req(req,"#test.p10")){
		printf("test pem write cert req file -- error!\n");
		return -1;
	}
	Cert_free(req);
	printf("test pem read & write cert req file -- ok\n");

	/* cross certificate pair */
	if((ccp=PEM_read_crtp("test.pem"))==NULL){
		printf("test pem read cross-cert-pair file -- error!\n");
		return -1;
	}
	if(PEM_write_crtp(ccp,"#test.ccp")){
		printf("test pem write cross-cert-pair file -- error!\n");
		return -1;
	}
	CertPair_free(ccp);
	printf("test pem read & write cross-cert-pair file -- ok\n");

	/* Private Key (RSA) */
	OK_set_passwd("abcde");
	if((rsa=PEM_read_rsaprv("test.pem"))==NULL){
		printf("test pem read RSA key file -- error!\n");
		return -1;
	}
	if(PEM_write_rsaprv(rsa,"#test.key")){
		printf("test pem write RSA key file -- error!\n");
		return -1;
	}
	Key_free((Key*)rsa);
	if((rsa=PEM_read_rsaprv("#test.key"))==NULL){
		printf("test pem read RSA key file (2) -- error!\n");
		return -1;
	}
	Key_free((Key*)rsa);
	OK_clear_passwd();
	printf("test pem read & write RSA key file -- ok\n");

	/* Private Key (DSA) */
	OK_set_passwd("abcde");
	if((dsa=PEM_read_dsaprv("dsakey.pem"))==NULL){
		printf("test pem read DSA key file -- error!\n");
		return -1;
	}
	if(PEM_write_dsaprv(dsa,"#dsakey.pem")){
		printf("test pem write DSA key file -- error!\n");
		return -1;
	}
	Key_free((Key*)dsa);
	if((dsa=PEM_read_dsaprv("#dsakey.pem"))==NULL){
		printf("test pem read DSA key file (2) -- error!\n");
		return -1;
	}
	Key_free((Key*)dsa);
	OK_clear_passwd();
	printf("test pem read & write DSA key file -- ok\n");

	/* DSA Parameter */
	if((dpm=PEM_read_dsaparam("dsapm.pem"))==NULL){
		printf("test pem read dsa param file -- error!\n");
		return -1;
	}
	if(PEM_write_dsaparam(dpm,"#dsapm.pem")){
		printf("test pem write dsa param file -- error!\n");
		return -1;
	}
	DSAPm_free(dpm);
	printf("test pem read & write DSA param file -- ok\n");

	/* Private Key (ECDSA) */
	OK_set_passwd("abcde");
	if((ecdsa=PEM_read_ecdsaprv("ecdsakey.pem"))==NULL){
		printf("test pem read ECDSA key file -- error!\n");
		return -1;
	}
	if(PEM_write_ecdsaprv(ecdsa,"#ecdsakey.pem")){
		printf("test pem write ECDSA key file -- error!\n");
		return -1;
	}
	Key_free((Key*)ecdsa);
	if((ecdsa=PEM_read_ecdsaprv("#ecdsakey.pem"))==NULL){
		printf("test pem read ECDSA key file (2) -- error!\n");
		return -1;
	}
	Key_free((Key*)ecdsa);
	OK_clear_passwd();
	printf("test pem read & write ECDSA key file -- ok\n");

	/* ECDSA Parameter */
	if((E=PEM_read_ecparam("ecdsapm.pem"))==NULL){
		printf("test pem read ecdsa param file -- error!\n");
		return -1;
	}
	if(PEM_write_ecparam(E,"#ecdsapm.pem")){
		printf("test pem write ecdsa param file -- error!\n");
		return -1;
	}
	ECPm_free(E);
	printf("test pem read & write ECDSA param file -- ok\n");

	return 0;
}

int test_pem_pkcs(){
	PKCS7 *p7;
	Key *p8;
	int i;

	/* read PEM PKCS#7 signed data */
	for(i=0;i<11;i++){
		if((p7=PEM_read_p7("testp7.pem"))==NULL){
			printf("test pem pkcs7 read file -- error!\n");
			return -1;
		}
		if(PEM_write_p7(p7,"testp7.pem")){
			printf("test pem pkcs7 write file -- error!\n");
			return -1;
		}
		if((i%5)==0)printf("test pem pkcs7 read & write file -- ok : %d\n",i);

		P7_free(p7);
	}

	/* read pem non-encrypted PKCS#8 file. */
	for(i=0;i<11;i++){
		if((p8=PEM_read_p8("testp8.pem"))==NULL){
			printf("test pem pkcs8 read file -- error!\n");
			return -1;
		}
		if(PEM_write_p8(p8,"testp8.pem")){
			printf("test pem pkcs8 write file -- error!\n");
			return -1;
		}
		if((i%5)==0)printf("test pem pkcs8 read & write file -- ok : %d\n",i);

		Key_free(p8);
	}

	/* read pem encrypted PKCS#8 file. */
	for(i=0;i<11;i++){
		OK_set_passwd("abcde");
		if((p8=PEM_read_p8enc("testp8e.pem"))==NULL){
			printf("test pem pkcs8 read file -- error!\n");
			return -1;
		}

		OK_set_p5_cry_algo(OBJ_P5_MD2DES + (i%6));

		if(PEM_write_p8enc(p8,"testp8e.pem")){
			printf("test pem pkcs8 write file -- error!\n");
			return -1;
		}
		OK_clear_passwd();

		if((i%5)==0) printf("test pem encrypted pkcs8 read & write file -- ok : %d\n",i);

		Key_free(p8);
	}
	return 0;
}

