/* pkcstestfc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_asn1.h"
#include "ok_x509.h"
#include "ok_pkcs.h"
#include "ok_tool.h"

#ifdef __WINDOWS__
#define P12_TESTF	".\\test.p12"
#define P12_TESTF2	".\\test2.p12"
#define P12_DSAF	".\\dsap.p12"
#define P12_DSAF2	".\\dsap2.p12"
#define P7S_TESTF	".\\test.p7b"
#define P7S_TESTF2	".\\test2.p7b"
#define P8_TESTF	".\\testp8.crtx"
#define P8_TESTF2	".\\testp8.crtx"
#define P8E_TESTF	".\\testp8e.crtx"
#define P8E_TESTF2	".\\testp8e.crtx"
#else
#define P12_TESTF	"./test.p12"
#define P12_TESTF2	"./test2.p12"
#define P12_DSAF	"./dsap.p12"
#define P12_DSAF2	"./dsap2.p12"
#define P7S_TESTF	"./test.p7b"
#define P7S_TESTF2	"./test2.p7b"
#define P8_TESTF	"./testp8.crtx"
#define P8_TESTF2	"./testp8.crtx"
#define P8E_TESTF	"./testp8e.crtx"
#define P8E_TESTF2	"./testp8e.crtx"
#endif

int test_pkcs_rwfile(){
	PKCS12	*p12,*dmy;
	PKCS7	*p7s,*dm2;
	int i;

	/* test PKCS#12 */
	for(i=0;i<11;i++){
		/* use default cipher setting.
		 * this means that cert encryption uses RC2 40bit,
		 * and key encryption uses 3DES 192bit.
		 * so actually, I can test all Pbe encryption & decryption
		 * algorithms :-)
		 */
		OK_set_passwd("abcde");
		if((p12=P12_read_file(P12_TESTF))==NULL){
			printf("test pkcs12 read file -- error!\n");
			return -1;
		}
		OK_set_passwd("abcde");
		if(P12_write_file(p12,P12_TESTF2)){
			printf("test pkcs12 write file -- error!\n");
			return -1;
		}
		OK_clear_passwd();

		/* test duplicate and FREE it! */
		dmy=P12_dup(p12);
		P12_free(p12);
		P12_free(dmy);

		if((i%5)==0)printf("test pkcs12 read & write file -- ok : %d\n",i);
	}

	/* test PKCS#12 (DSA key & cert version) */
	for(i=0;i<11;i++){
		/* use default cipher setting.
		 * this means that cert encryption uses RC2 40bit,
		 * and key encryption uses 3DES 192bit.
		 * so actually, I can test all Pbe encryption & decryption
		 * algorithms :-)
		 */
		OK_set_passwd("abcde");
		if((p12=P12_read_file(P12_DSAF))==NULL){
			printf("test pkcs12 (DSA) read file -- error!\n");
			return -1;
		}
		OK_set_passwd("abcde");
		if(P12_write_file(p12,P12_DSAF2)){
			printf("test pkcs12 (DSA) write file -- error!\n");
			return -1;
		}
		OK_clear_passwd();

		/* test duplicate and FREE it! */
		dmy=P12_dup(p12);
		P12_free(p12);
		P12_free(dmy);

		if((i%5)==0)printf("test pkcs12 (DSA) read & write file -- ok : %d\n",i);
	}

	/* test PKCS#7 (p7b) */
	for(i=0;i<11;i++){
		if((p7s=P7s_read_file(P7S_TESTF))==NULL){
			printf("test pkcs7 read file -- error!\n");
			return -1;
		}
		
		dm2=P7_dup(p7s);
		if(P7s_write_file(dm2,P7S_TESTF2)){
			printf("test pkcs7 write file -- error!\n");
			return -1;
		}

		/* test duplicate and FREE it! */
		P7_free(p7s);
		P7_free(dm2);

		if((i%5)==0)printf("test pkcs7b read & write file -- ok : %d\n",i);
	}
	return 0;
}

int test_pkcs_p7(){
	PKCS12	*p12;
	PKCS7	*p7s,*p7m,*dm2;
	unsigned char *der,*ret;
	int i,j;

	/* test PKCS#7 (p7s) */
	for(i=0;i<11;i++){
		char *txt = "This is sample txt file for PKCS#7 Signed Data...!#$%&'()==@:;testTEST :-)";

		OK_set_passwd("abcde");
		if((p12=P12_read_file(P12_TESTF2))==NULL){
			printf("test pkcs7 Signed : read p12 file -- error!\n");
			return -1;
		}
		OK_clear_passwd();

		j = strlen(txt);
		if((p7s= P7s_get_signed(p12, txt, j, default_p7s_digest_algo))==NULL){
			printf("test pkcs7 Signed : P7s_get_signed -- error!\n");
			return -1;
		}
		if(P7s_verify_signed(p7s,NULL,0)){
			printf("test pkcs7 Signed : P7s_verify_signed (1) -- error!\n");
			return -1;
		}

		dm2= P7_dup(p7s);
		P7_free(p7s);

		der=P7_signed_toDER(dm2,NULL,&j);

		if((p7s=ASN1_read_p7s(der))==NULL){
			printf("test pkcs7 Signed : ASN1_read_p7s -- error!\n");
			return -1;
		}

		if(P7s_verify_signed(p7s,NULL,0)){
			printf("test pkcs7 Signed : P7s_verify_signed (2) -- error!\n");
			return -1;
		}

		FREE(der);
		P7_free(p7s);
		P7_free(dm2);
		P12_free(p12);
		if((i%5)==0)printf("test pkcs7 Signed gen & read & vfy -- ok : %d\n",i);
	}

	/* test PKCS#7 (p7m) */
	for(i=0;i<11;i++){
		char *txt = "This is sample txt file for PKCS#7 Enveloped Data...!#$%&'()==@:;testTEST :-)";
		Cert *cert;
		Key *key;

		OK_set_passwd("abcde");
		if((p12=P12_read_file(P12_TESTF2))==NULL){
			printf("test pkcs7 Signed : read p12 file -- error!\n");
			return -1;
		}
		OK_clear_passwd();

		j = strlen(txt)+1;
		if((p7m=P7m_encrypt_enveloped((PKCS7*)p12,txt,j))==NULL){
			printf("test pkcs7 Envelope : P7m_encrypt_enveloped -- error!\n");
			return -1;
		}

		dm2= P7_dup(p7m);
		P7_free(p7m);

		der=P7_envelope_toDER(dm2,NULL,&j);

		if((p7m=ASN1_read_p7env(der))==NULL){
			printf("test pkcs7 Envelope : ASN1_read_p7env -- error!\n");
			return -1;
		}

		cert=P12_get_usercert(p12);
		key=P12_get_privatekey(p12);
		if((ret=P7m_decrypt_enveloped(p7m,cert,key))==NULL){
			printf("test pkcs7 Envelope : P7m_decrypt_enveloped -- error!\n");
			return -1;
		}

		if(strcmp(txt,ret)){
			printf("test pkcs7 Envelope : encrypt & decrypt -- error!\n");
			return -1;
		}
		FREE(der);
		FREE(ret);
		P7_free(p7m);
		P7_free(dm2);
		P12_free(p12);
		if((i%5)==0)printf("test pkcs7 Envelope : encrypt & decrypt -- ok : %d\n",i);
	}
	return 0;
}

int test_pkcs_p8(){
	Key *p8;
	int i;

	/* read non-encrypted PKCS#8 file. */
	for(i=0;i<11;i++){
		if((p8=P8_read_file(P8_TESTF))==NULL){
			printf("test pkcs8 read file -- error!\n");
			return -1;
		}
		if(P8_write_file(p8,P8_TESTF2)){
			printf("test pkcs8 write file -- error!\n");
			return -1;
		}
		if((i%5)==0)printf("test pkcs8 read & write file -- ok : %d\n",i);

		Key_free(p8);
	}

	/* read encrypted PKCS#8 file. */
	for(i=0;i<11;i++){
		OK_set_passwd("abcde");
		if((p8=P8enc_read_file(P8E_TESTF))==NULL){
			printf("test pkcs8 read file -- error!\n");
			return -1;
		}

		OK_set_p5_cry_algo(OBJ_P5_MD2DES + (i%6));

		if(P8enc_write_file(p8,P8E_TESTF2)){
			printf("test pkcs8 write file -- error!\n");
			return -1;
		}
		Key_free(p8);
		if((p8=P8enc_read_file(P8E_TESTF2))==NULL){
			printf("test pkcs8 read file -- error!\n");
			return -1;
		}
		OK_clear_passwd();

		if((i%5)==0) printf("test encrypted pkcs8 read & write file -- ok : %d\n",i);

		Key_free(p8);
	}

	return 0;
}
