/* aiconf.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include "aiconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef __WINDOWS__
# undef ULONG
# include <windows.h>
# include <winreg.h>
# include <tchar.h>
#endif

#include "ok_asn1.h"
#include "ok_pem.h"
#include "ok_rsa.h"
#include "ok_x509.h"
#include "ok_pkcs.h"

#include "ok_ca.h"

char* conf_get_section(char *fname,char *begin,char *end){
  char *buf,*ret,*sp,*ep;
  int i,err=-1;

  if((buf=get_file2buf(fname,&i))==NULL) return NULL;

  if((sp=strstr(buf,begin))==NULL){
    OK_set_error(ERR_ST_CA_BADCONF,ERR_LC_CONF,ERR_PT_CONF,NULL);
    goto done;
  }
  if((ep=strstr(sp,end))==NULL){
    OK_set_error(ERR_ST_BADFORMAT,ERR_LC_CONF,ERR_PT_CONF,NULL);
    goto done;
  }
  *ep=0;
  
  if((STRDUP(ret,sp))==NULL){
    OK_set_error(ERR_ST_STRDUP,ERR_LC_CONF,ERR_PT_CONF,NULL);
    goto done;
  }

done:
  AIFREE(buf);
  return ret;
}

#ifdef __WINDOWS__
int get_store_path(char *ret,int max){
	HKEY k1=NULL, k2=NULL;
	char str[256];
	DWORD i,j;
	int ok = -1;

	if(RegOpenKey(HKEY_CURRENT_USER,_T("Software"),&k1) != ERROR_SUCCESS) goto done;
	if(RegOpenKey(k1,_T("Usagi Application"),&k2) != ERROR_SUCCESS) goto done;

	j = sizeof(str);
	memset(str,0,j);
	if(RegQueryValueEx(k2,"StorePath",NULL,&i,(unsigned char*)str,&j) != ERROR_SUCCESS) goto done;
	strncpy(ret,str,max);

	ok = 0;
done:
	if(k1) RegCloseKey(k1);
	if(k2) RegCloseKey(k2);
	return ok;
}

int get_aiconf_path(char *ret,int max){
	HKEY k1=NULL, k2=NULL;
	char str[256];
	DWORD i,j;
	int ok = -1;

	if(RegOpenKey(HKEY_CURRENT_USER,_T("Software"),&k1) != ERROR_SUCCESS) goto done;
	if(RegOpenKey(k1,_T("Usagi Application"),&k2) != ERROR_SUCCESS) goto done;

	j = sizeof(str);
	memset(str,0,j);
	if(RegQueryValueEx(k2,"AiconfPath",NULL,&i,(unsigned char*)str,&j) != ERROR_SUCCESS) goto done;
	strncpy(ret,str,max);

	ok = 0;
done:
	if(k1) RegCloseKey(k1);
	if(k2) RegCloseKey(k2);
	return ok;
}

int get_install_path(char *ret,int max){
	HKEY k1=NULL, k2=NULL;
	char str[256];
	DWORD i,j;
	int ok = -1;

	if(RegOpenKey(HKEY_CURRENT_USER,_T("Software"),&k1) != ERROR_SUCCESS) goto done;
	if(RegOpenKey(k1,_T("Usagi Application"),&k2) != ERROR_SUCCESS) goto done;

	j = sizeof(str);
	memset(str,0,j);
	if(RegQueryValueEx(k2,"InstallPath",NULL,&i,(unsigned char*)str,&j) != ERROR_SUCCESS) goto done;
	strncpy(ret,str,max);

	ok = 0;
done:
	if(k1) RegCloseKey(k1);
	if(k2) RegCloseKey(k2);
	return ok;
}
#endif /* __WINDOWS__ */
