/* asn1view.c */
/*
 * Copyright (C) 1998-2001
 *  Akira Iwata Laboratory. 
 *  Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usagi@mars.elcom.nitech.ac.jp)
 * And if you want to contact us, send e-mail to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.
 * If you want to use aicrypto library and CA applications code in product,
 * should be e-mail to Akira Iwata Laboratory (wakayama@elcom...).
 * 
 * Please note that MD2 and MD5 include RSA Data Security, Inc. LICENSE.
 * Those are besed on RFC1319 and RFC1321 document. And copyright distribution
 * is following in ok_md2.h ok_md5.h .
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_asn1.h"
#include "ok_pem.h"
#include "ok_x509.h"
#include "ok_tool.h"

void usage(void);
void options(void);
int print_pkcs12(char *fname);

/*-----------------------------------------
  ASN.1 Viewer
-----------------------------------------*/
int main(int argc,char **argv){
  unsigned char *der;

  if(argc == 1){
    usage();
    exit(1);
  }

#ifdef __WINDOWS__
  OK_set_pwd_dosmode(1);
#endif

  if(der = PEM_read_cert_2der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
  }
  if(der = PEM_read_crl_2der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
  }
  if(der = PEM_read_rsaprv_2der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
  }
  if(der = PEM_read_dsaprv_2der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
  }
  if(der = PEM_read_ecdsaprv_2der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
  }
  if(der = PEM_read_req_2der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
  }
  if(der = PEM_read_p7_2der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
    return 0;
  }
  if(der = PEM_read_p8_2der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
    return 0;
  }
  if(der = PEM_read_p8enc_2der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
    return 0;
  }

  if(print_pkcs12(argv[1]))
      return 1;

  if(der = ASN1_read_der(argv[1])){
    OK_ASN1_print(der);
    AIFREE(der);
  }
  return 0;
}

void usage(void){
  printf("usage: asn1view file\n");
  printf("\t file is PEM or P12 or DER\n");
}





