/* certreq.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_rand.h"
#include "ok_asn1.h"
#include "ok_pem.h"
#include "ok_tool.h"
#include "ok_uconv.h"

#include "ok_ca.h"

#ifndef	AICONFIG
#define	AICONFIG "aica.cnf"
#endif

int get_key_pairs(Key **pub,Key **prv,int size,int type);
char *input_subject(CertDN *dn,char *prompt);
int write_files(Cert *req,Key *prv);
int req_config(char *fname);

void options(int argc,char **argv);
void usage(void);

int  type=0;
int  keysize=512;
int  keytype=KEY_RSA_PUB;
char pass[32] ="";
char keyf[256]="newkey.key";
char reqf[256]="newreq.p10";
char conf[256]=AICONFIG;


int main(int argc,char **argv){
  Req	*req=NULL;
  Key	*pub,*prv;
  int	i,err=-1;

  pub=prv=NULL;
  options(argc,argv);
  RAND_init();

#ifdef __WINDOWS__
  get_aiconf_path(conf,256);
#endif

  if(req_config(conf)){
      printf("cannot find a config file : %s\n",conf);
      goto done;
  }

  if((req=Req_new())==NULL) goto done;

  if(get_key_pairs(&pub,&prv,keysize,keytype)) goto done;

  req->pubkey_algo = keytype;
  req->pubkey = pub;

  if((req->subject=input_subject(&req->subject_dn,NULL))==NULL) goto done;
/*print_dir(&req->subject_dn);*/

  switch(keytype){
  case KEY_RSA_PUB:   i=OBJ_SIG_SHA1RSA; break;
  case KEY_DSA_PUB:   i=OBJ_SIG_SHA1DSA; break;
  case KEY_ECDSA_PUB: i=OBJ_SIG_SHA1ECDSA; break;
  }
  OK_set_cert_sig_algo(i);
  if((req->der = Req_toDER(req,prv,NULL,&i))==NULL) goto done;

  Req_print(req);

  if(*pass) OK_set_passwd(pass);

  if(write_files(req,prv)) goto done;
  err=0;

done:
  RAND_cleanup();
  free_u2j_table();

  Req_free(req);
  Key_free(prv);
  if(err) printf("%s\n",CA_get_errstr());
  return err;
}

/*-----------------------------------------
  usage and option check
-----------------------------------------*/
void options(int argc,char **argv){
  int	i;

#ifdef __WINDOWS__
  OK_set_pwd_dosmode(1);
#endif

  for(i=1;i<argc;i++){
    if(!strcmp("-conf",argv[i])){
      i++;
      if(i<argc) strncpy(conf,argv[i],254);
    }else if(!strcmp("-req",argv[i])){
      i++;
      if(i<argc) strncpy(reqf,argv[i],254);
    }else if(!strcmp("-key",argv[i])){
      i++;
      if(i<argc) strncpy(keyf,argv[i],254);
    }else if(!strcmp("-p",argv[i])){
      i++;
      if(i<argc) strncpy(pass,argv[i],30);
    }else if(!strcmp("-der",argv[i])){
      type++;

    }else if(!strcmp("-algo",argv[i])){
      i++;
      if(i>=argc) continue;
      if(!strcmp("dsa",argv[i])){
	keytype = KEY_DSA_PUB;
      }else if(!strcmp("ecdsa",argv[i])){
	keysize = 192;
	keytype = KEY_ECDSA_PUB;
      }
    }else if(!strcmp("-size",argv[i])){
      i++;
      if(i<argc) keysize=atoi(argv[i]);
    }else if(!strcmp("-noenc",argv[i])){
      OK_set_pem_cry_algo(0);   /* don't encrypto PEM */

    }else{
      printf("option error!\n");
      usage();
      exit(0);
    }
  }
}

void usage(void){
  printf("usage: certreq [outputs]\n");
  printf("\t-conf path: set aica config path\n");
  printf("\t-req file : output request named \"file\".\n");
  printf("\t-key file : output key named \"file\".\n");
  printf("\t-algo alg : prvkey algorithm ... rsa,dsa,ecdsa [default:rsa]\n");
  printf("\t-size num : prvkey size [default:512bit]\n");
  printf("\t-p passwd : password for output private key.\n");
  printf("\t-noenc    : save private key without encryption\n");
  printf("\t-der      : set output as DER format [default:PEM]\n");
}

int write_files(Cert *req,Key *prv){

  if(type){  if(ASN1_write_der(req->der,reqf)) goto error; }
  else{      if(PEM_write_req(req,reqf)) goto error; }

  switch(prv->key_type){
  case KEY_RSA_PRV:
    if(PEM_write_rsaprv((Prvkey_RSA*)prv,keyf)) goto error;
    break;
  case KEY_DSA_PRV:
    if(PEM_write_dsaprv((Prvkey_DSA*)prv,keyf)) goto error;
    break;
  case KEY_ECDSA_PRV:
    if(PEM_write_ecdsaprv((Prvkey_ECDSA*)prv,keyf)) goto error;
    break;
  }

  return 0;
error:
  return -1;
}


