/* certview.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */
#include "aiconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_asn1.h"
#include "ok_rsa.h"
#include "ok_dsa.h"
#include "ok_ecdsa.h"
#include "ok_x509.h"
#include "ok_pem.h"
#include "ok_pkcs.h"
#include "ok_tool.h"


char *pass = NULL;
char *in[8];
int  fn;

void usage(void);
void options(int argc,char **argv);

/* key.c */
void key_print_dsaparam(DSAParam *pm);
void key_print_ecparam(ECParam *pm);

/* main */
int main(int argc,char **argv){
	unsigned char *der,i;
	CertPair *cp;
	Cert *ct;
	CRL *crl;
	Key *key;
	PKCS7 *p7;
	PKCS12 *p12;
	DSAParam *pm;
	ECParam *E;

	if(argc == 1){
		usage();
		exit(1);
	}

	options(argc,argv);

	OK_set_prompt("Input Password: ");
	if(pass) OK_set_passwd(pass);

	for(i=0;i<fn;i++){
		if(!fopen(in[i],"r")) continue;

		if(ct = Cert_read_file(in[i])){
			Cert_print(ct);
			Cert_free(ct);
			continue;
		}
		if(ct = Req_read_file(in[i])){
			Cert_print(ct);
			Cert_free(ct);
			continue;
		}
		if(crl = CRL_read_file(in[i])){
			CRL_print(crl);
			CRL_free(crl);
			continue;
		}
		if(cp = CertPair_read_file(in[i])){
			CertPair_print(cp);
			CertPair_free(cp);
			continue;
		}
		if(key = (Key*)PEM_read_rsaprv(in[i])){
			Key_print(key);
			Key_free(key);
			continue;
		}
		if(key = (Key*)PEM_read_dsaprv(in[i])){
			Key_print(key);
			Key_free(key);
			continue;
		}
		if(key = (Key*)PEM_read_ecdsaprv(in[i])){
			Key_print(key);
			Key_free(key);
			continue;
		}
		if(pm = PEM_read_dsaparam(in[i])){
			key_print_dsaparam(pm);
			DSAPm_free(pm);
			continue;
		}
		if(E = PEM_read_ecparam(in[i])){
			key_print_ecparam(E);
			ECPm_free(E);
			continue;
		}
		if(p7 = P7b_read_file(in[i])){
			P7_print(p7);
			P7_free(p7);
			continue;
		}
		if(p12 = P12_read_file(in[i])){
			P12_print(p12);
			P12_free(p12);
			continue;
		}
		if(key = (Key*)P8_read_file(in[i])){
			Key_print(key);
			Key_free(key);
			continue;
		}
		if(key = (Key*)P8enc_read_file(in[i])){
			Key_print(key);
			Key_free(key);
			continue;
		}
		if(der = ASN1_read_der(in[i])){
			if(key = (Key*)ASN1_read_rsaprv(der)){
				Key_print(key);
				Key_free(key);
				continue;
			}
			if(key = (Key*)ASN1_read_dsaprv(der)){
				Key_print(key);
				Key_free(key);
				continue;
			}
			if(key = (Key*)ASN1_read_ecdsaprv(der)){
				Key_print(key);
				Key_free(key);
				continue;
			}
			if(pm = ASN1_read_dsaparam(der,0)){
				key_print_dsaparam(pm);
				DSAPm_free(pm);
				continue;
			}
			if(E = ASN1_read_ecparam(der)){
				key_print_ecparam(E);
				ECPm_free(E);
				continue;
			}
		}
	}
	return 0;
}

void usage(void){
	printf("usage: certview [options] file1 file2 ..\n");
	printf("  options are ..\n");
	printf("    -p passwd    : password for key & pkcs#12 file.\n");
	printf("  file type is ..\n");
	printf("    Certificate  : *.cer *.p7b *.p12 *.pem [x509(DER),PKCS#7,PKCS#12,PEM]\n");
	printf("    CRL          : *.crl *.p7b *.p12 *.pem [x509(DER),PKCS#7,PKCS#12,PEM]\n");
	printf("    Private Key  : *.p12 *.crtx *.pem [PKCS#12,PKCS#8,PEM]\n");
	printf("    Key Parameter: *.pem [dsa,ecdsa]\n");
	printf("    Cert Reqest  : *.p10 *.pem [PKCS#10,PEM]\n");
	printf("    XCertPair    : *.ccp *.pem [x509(DER),PEM]\n");
}

void options(int argc,char **argv){
	int i;

#ifdef __WINDOWS__
	OK_set_pwd_dosmode(1);
#endif

	for(i=1,fn=0;i<argc;i++){
		if(argv[i][0]=='-'){
			if(!strcmp(argv[i],"-p")){
				i++;
				if(i<argc) pass = argv[i];
			}else{
				printf("option error!\n");
				usage();
				exit(0);
			}
		}else{
			in[fn] = strdup(argv[i]);
			fn++;
		}
	}
}
