/* ca.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_x509.h"
#include "ok_x509ext.h"
#include "ok_pkcs.h"

#include "ok_ca.h"


/*-----------------------------------------
  struct CA alloc & free
-----------------------------------------*/
CA *CA_new(void){
	CA *ret;

	if((ret=(CA*)AIMALLOC(sizeof(CA)))==NULL){
		OK_set_error(ERR_ST_MEMALLOC,ERR_LC_CA,ERR_PT_CA,NULL);
		return NULL;
	}
	memset(ret,0,sizeof(CA));
	return ret;
}

void CA_free(CA *ca){
	int i;
	if(ca==NULL) return;

	P12_free(ca->p12);

	if(ca->subject) AIFREE(ca->subject);
	if(ca->issuer)  AIFREE(ca->issuer);
	if(ca->der)		AIFREE(ca->der);
	if(ca->policy)	AIFREE(ca->policy);
	if(ca->ca_name)	AIFREE(ca->ca_name);
	if(ca->ca_path) AIFREE(ca->ca_path);
	if(ca->m_tag)	AIFREE(ca->m_tag);

	for(i=0; i<ca->sbj_num; i++)
		if(ca->sbjList[i]) FREE(ca->sbjList[i]);

	/* ca->cert & ca->prvkey is just pointer
	 * to PKCS#12 bags. Don't need to free them.
	 */

	CRL_free(ca->crl);

	CertExt_free_all(ca->ext);
	CertStat_free_all(ca->stat);

	Prof_cert_free_all(ca->profList);
	Prof_crl_free_all(ca->crlpList);

	AIFREE(ca);
}

/*-----------------------------------------
  struct CertStat alloc & free
-----------------------------------------*/
CertStat *CertStat_new(void){
	CertStat *ret;

	if((ret=(CertStat*)AIMALLOC(sizeof(CertStat)))==NULL){
		OK_set_error(ERR_ST_MEMALLOC,ERR_LC_CA,ERR_PT_CA+2,NULL);
		return NULL;
	}
	memset(ret,0,sizeof(CertStat));
	return ret;
}

void CertStat_free(CertStat *cs){
	if(cs==NULL) return;
	if(cs->subject) AIFREE(cs->subject);
	AIFREE(cs);
}

void CertStat_free_all(CertStat *cs){
	CertStat *next;
	while(cs){
		next=cs->next;
		CertStat_free(cs);
		cs=next;
	}
}

