/* ca_err.c */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ok_ca.h"

/*-----------------------------------------
  CA get error string
-----------------------------------------*/
char *ca_get_err_location(int err){
	static char buf[64],tmp[16];
	int loc,point = err>>8;

	loc  = err & 0xff;
	*buf = 0;

	switch(loc){
	case ERR_LC_CA:
		strcat(buf,"CA:");
		switch(point&0xf0){
		case ERR_PT_CA: strcat(buf,"CA"); break;
		case ERR_PT_CACRL: strcat(buf,"CACRL"); break;
		case ERR_PT_CAFILE: strcat(buf,"CAFILE"); break;
		case ERR_PT_CAASN1: strcat(buf,"CAASN1"); break;
		case ERR_PT_CALCERTS: strcat(buf,"CALCERTS"); break;
		case ERR_PT_CANEW: strcat(buf,"CANEW"); break;
		case ERR_PT_CAPRINT: strcat(buf,"CAPRINT"); break;
		case ERR_PT_CASIGN: strcat(buf,"CASIGN"); break;
		case ERR_PT_CATOOL: strcat(buf,"CATOOL"); break;
		case ERR_PT_CACONF: strcat(buf,"CACONF"); break;
		case ERR_PT_CAPOL: strcat(buf,"CAPOL"); break;
		}
		sprintf(tmp,"(%d)",point&0xf);
		strcat(buf,tmp);
		break;

	case ERR_LC_REQ:
		strcat(buf,"REQ:");
		switch(point&0xf0){
		case ERR_PT_REQ: strcat(buf,"REQ"); break;
		}
		sprintf(tmp,"(%d)",point&0xf);
		strcat(buf,tmp);
		break;

	case ERR_LC_CONF:
		strcat(buf,"CONF:");
		switch(point&0xf0){
		case ERR_PT_CONF: strcat(buf,"CONF"); break;
		}
		sprintf(tmp,"(%d)",point&0xf);
		strcat(buf,tmp);
		break;

	default:
		strcat(buf,get_err_location(err));
		break;
	}
	return buf;
}

char *ca_get_err_type(int err){
	char *ret;
	switch(err){
	case ERR_ST_CA_NOCSTAT: ret="no such cert status"; break;
	case ERR_ST_CA_NORSTAT: ret="no such req status"; break;
	case ERR_ST_CA_NOEXT: ret="no such extensions"; break;
	case ERR_ST_CA_NOPROF: ret="no such profile name"; break;
	case ERR_ST_CA_BADCONF: ret="bad config file"; break;
	case ERR_ST_CA_BADCSV: ret="bad csv file"; break;
	case ERR_ST_CA_BADSN: ret="bad serial number"; break;
	case ERR_ST_CA_CTREVOKED: ret="a certificate was revoked"; break;
	case ERR_ST_CA_CTEXPIRED: ret="a certificate was expired"; break;
	case ERR_ST_CA_CANCELED: ret="operation is canceled"; break;
	case ERR_ST_CA_POLMIS: ret="cert subject is unmatched with CA policy"; break;
	default: ret=get_err_type(err); break;
	}
	return ret;
}

char *CA_get_errstr(){
	static char buf[128];
	ULONG errnum = OK_get_error();

	sprintf(buf,"[aica] error:0x%x:%s:%s",
		errnum,
		ca_get_err_location(errnum>>16),
		ca_get_err_type(errnum&0xffff));
	return buf;
}


