/* ok_aica.h */
/*
 * Copyright (C) 1998-2002
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef __OK_AICA_H__
#define __OK_AICA_H__

#include "ok_ca.h"

#ifdef  __cplusplus
extern "C" {
#endif

#define NS_CRL		"http://sample.crl.co.jp/"
#define NS_COMMENT	"AiCA Comment."
#define NS_CERTTYPE	0xb0

#define	AICA_OP_ISSUE	100
#define	AICA_OP_SIGN	101
#define	AICA_OP_RESIGN	102
#define	AICA_OP_CRL		110
#define	AICA_OP_PRINT	120
#define	AICA_OP_LIST	121
#define	AICA_OP_REVOKE	130
#define	AICA_OP_CSV		140
#define	AICA_OP_NEW		150
#define	AICA_OP_PFSET	160
#define	AICA_OP_PFEXT	170
#define	AICA_OP_PROF	180

/* ca_new.c */
int CA_init_Info(CA *ca);
int CA_new_with_p12(CA *ca);
int CA_new_generate(CA *ca,int size,int type);
int CA_update_P12andInfo(CA *ca,int flg);
int CA_set_defaulttags(CA *ca, CertDN *dir);

/* ca_pol.c */
int CA_check_policy(CA *ca,Cert *ct);
int CA_set_policy(CA *ca,char *policy);

/* ca_conf.c */
int config_set_capath(char *str);
int CA_read_config(char *fname);
int CA_set_ca_path(CA *ca,char *path);
int CA_set_lib_path(CA *ca);

/* ca_print.c */
void CA_print(CA *ca,char *pfname,int pfall);
void CA_certstat_print(CA *ca,char *pfname);

/* aica_file.c */
CA *CA_info_read(char *path);
int CA_info_write(CA *ca);
int CA_p12_read(CA *ca);
int CA_write_cert(CA *ca,Cert *cert,char *fname);
int CA_write_crl(CA *ca,char *fname);

/* aica_sign.c */
int Sign_do(CA *ca,Cert *req);
int Sign_certext(CA *ca,Cert *ct);
int Sign_before_after(CA *ca,Cert *ct);
int CA_do_revocation(CA *ca,int snum);

/* aica_lcerts.c */
int CA_issue_alotof_certificates(CA *ca,char *fname);

/* aica_setext.c */
int CA_set_extensions(CA *ca,int extoid);
int ca_ask_onoff(char *txt,int on);
int ca_ask_yesno(char *txt,int yes);
int ca_ask_num(char *txt,int num);
int ca_ask_comment(char *txt,char *buf, int max);

/* aica_prof.c */
int CA_do_profile(CA* ca, int mode);

/* aiconf.c */
char* conf_get_section(char *fname,char *begin,char *end);
#ifdef __WINDOWS__
int get_store_path(char *ret,int max);
int get_aiconf_path(char *ret,int max);
int get_install_path(char *ret,int max);
#endif /* __WINDOWS__ */


/* req.c */
void config_set_defdir(char *str);
int req_config(char *fname);
int get_key_pairs(Key **pub,Key **prv,int size,int type);
char *input_subject(CertDN *dn,char *prompt);
ExtGenNames *input_ipaddress();
OtherName *input_othername();
ExtGenNames *input_gennames(char *prompt);
int ca_set_defdir(CertDN *dn);

#ifdef  __cplusplus
}
#endif

#endif /* __OK_AICA_H__ */
