/* ok_ca.h */
/*
 * Copyright (C) 1998-2001
 * Akira Iwata & Takuto Okuno
 * Akira Iwata Laboratory,
 * Nagoya Institute of Technology in Japan.
 *
 * All rights reserved.
 *
 * This software is written by Takuto Okuno(usapato@anet.ne.jp)
 * And if you want to contact us, send an email to Kimitake Wakayama
 * (wakayama@elcom.nitech.ac.jp)
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this software must
 *    display the following acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *    Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 * 4. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Akira Iwata Laboratory,
 *     Nagoya Institute of Technology in Japan (http://mars.elcom.nitech.ac.jp/)."
 *
 *   THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY.
 *   AKIRA IWATA LABORATORY DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *   SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
 *   IN NO EVENT SHALL AKIRA IWATA LABORATORY BE LIABLE FOR ANY SPECIAL,
 *   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 *   FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 *   NEGLIGENCE OR OTHER TORTUOUS ACTION, ARISING OUT OF OR IN CONNECTION
 *   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef __OK_CA_H__
#define __OK_CA_H__

#include <time.h>

#include "ok_caerr.h"
#include "ok_x509.h"
#include "ok_x509ext.h"
#include "ok_pkcs.h"

#ifdef  __cplusplus
extern "C" {
#endif

/* certificate state structure */
/*
 * this structure has certificate information used for
 * CA operations. FILE pointer and its position indecates
 * *.cts (certificates) file can be used.
 */
typedef struct cetificate_state CertStat;
struct cetificate_state{
	int	state;
	int	serialNum;
	char	*subject;
	struct tm notBefore;
	struct tm notAfter;
	struct tm revokedDate;
	CertStat	*next;
	CertStat	*prev;

	FILE	*fp;
	fpos_t	pos;
};

/* certificate request state structure */
/*
 * this structure has CSR and private key information used
 * for CA operations. FILE pointer and its position indecates
 * *.rqs (requests) and *.kys (private keys) file can be used.
 */
typedef struct request_state ReqStat;
struct request_state{
	int		serialNum;
	char	*subject;
	struct tm date;
	ReqStat	*next;
	ReqStat	*prev;

	FILE	*rfp,*kfp;
	fpos_t	rpos,kpos;
};

typedef struct certificate_authority CA;
typedef struct certificate_profile CertProf;
typedef struct crl_profile CRLProf;

struct certificate_profile
{
	int		ver;
	int		isCross;
	CA		*pCA;

	CertStat *stat;
	char	*ca_name;
	char	*name;

	FILE	*fp;
	fpos_t	pos;

	CertExt	*ext;

	int		serialNum;
	int		cert_days;
	int		cert_ver;
	int		keylong;
	int		sigtype;

	int		bgtype,edtype;

	struct tm ct_begin;
	struct tm ct_end;

	unsigned char *der;

	CertProf	*next;
};

struct crl_profile
{
	int		ver;
	CA		*pCA;
	char	*ca_name;
	char	*name;

	CertExt	*ext;

	int		crlNum;
	int		crl_days;
	int		crl_ver;
	int		sigtype;

	unsigned char *der;

	CRLProf	*next;
};

struct certificate_authority{
	PKCS12	*p12;
	CA		*next;

	char	*subject;
	char	*issuer;

	Cert	*cert;
	Key		*prvkey;
	CRL		*crl;

	CertExt *ext;

	long	serialNum;
	int		cert_ver;
	int		cert_days;
	int		crl_ver;
	int		crl_num;
	int		crl_days;

	CertStat	*stat;

	/* der */
	unsigned char *der;

	FILE	*fp;
	fpos_t	pos;

	unsigned char	*policy;

	int		ver;	/* CA version */
	int		sigtype;
	int		keylong;
	int		sbj_ft;
	int		plginsign;
	int		ca_update;
	char	*ca_name;
	char	*ca_path;
	char	*lib_path;

	int		sbj_num;
	char	*m_tag;
	char	*sbjList[16];

	CertProf *cprof;	/* current active profile */
	CertProf *profList;
	CRLProf *lprof;	/* current active profile */
	CRLProf *crlpList;
};


#define	STAT_OK		0
#define STAT_CA		0x1
#define	STAT_EXPIRED	0x2
#define	STAT_REVOKED	0x4
#define STAT_XCERTFWD	0x10
#define STAT_XCERTREV	0x20
#define	STAT_HAVEKEY	0x100

#define STAT_RR_unspecified				0x00000
#define STAT_RR_keyCompromise			0x10000
#define STAT_RR_cACompromise			0x20000
#define STAT_RR_affiliationChanged		0x30000
#define STAT_RR_superseded				0x40000
#define STAT_RR_cessationOfOperation	0x50000
#define STAT_RR_certificateHold			0x60000
#define STAT_RR_removeFromCRL			0x80000
#define STAT_RR_privilegeWithdrawn		0x90000
#define STAT_RR_aaCompromise			0xa0000

/* ca.c */
CA *CA_new(void);
void CA_free(CA *ca);

CertStat *CertStat_new(void);
void CertStat_free(CertStat *cs);
void CertStat_free_all(CertStat *cs);

/* ca_asn1.c */
unsigned char *CA_toDER(CA *ca,unsigned char *buf,int *ret_len);
int CA_DER_extns(CA *ca,unsigned char *ret,int *ret_len);
int CA_DER_castat(CA *ca,unsigned char *ret,int *ret_len);
int CA_estimate_der_size(CA *ca);
char *CA_get_defsubstr(CA *ca);

CA *ASN1_CA_info(unsigned char *der);
int ASN1_CA_castat(CA *ca,unsigned char *in);
int ASN1_CA_certstat(CA *ca,unsigned char *in);

/* ca_tool.c */
int CA_now_certnum(CA *ca);
int CA_can_serialNum(CA *ca,Cert *req,int sn);
int CA_can_resign(CA *ca,Cert *ct);
int CA_expire_check(CA *ca);
int CA_key2sigalgo(int type);

int CA_add_cert(CA *ca,Cert *ct);
int CA_update_cert(CA *ca,Cert *ct);
int CA_add_certfile(CA *ca,Cert *ct);
void CA_delete_stat(CertStat *st);
CertStat *CA_find_stat(CA *ca,int serial);
CertStat *CA_find_statstr(CA *ca,char *subject);
CertProf *CA_find_prof_sn(CA *ca,int serial);
CertProf *CA_find_prof_sbj(CA *ca,char *subject);

/* ca_crl */
int CA_cert_revoke(CA *ca,int revoke,int reason);
int CA_set_CRL(CA *ca);
int CA_crl_last_next(CA *ca,CRL *crl);
int CA_crl_set_revoked(CA *ca,CRL *crl,int *rev_num);
int CA_crl_set_ext(CA *ca,CRL *crl);

/* ca_err.c */
char *ca_get_err_location(int err);
char *ca_get_err_type(int err);
char *CA_get_errstr();

/* prof.c */
CertProf *Prof_cert_new(CA *ca,char *name);
void Prof_cert_free(CertProf *cpf);
void Prof_cert_free_all(CertProf *top);

CRLProf *Prof_crl_new(CA *ca,char *name);
void Prof_crl_free(CRLProf *cpf);
void Prof_crl_free_all(CRLProf *top);

/* prof_tmpl.c */
int Prof_cert_settmpl(CA *ca, char *name);
int Prof_crl_settmpl(CA *ca, char *name);
int get_tmpl_path(CA *ca,char *buf,int max);

/* prof_asn1.c */
int ASN1_ctprof_info(CA *ca,unsigned char *in);
int ASN1_ctprof_stat(CA *ca, unsigned char *in);
int ASN1_ctprof_ctstat(CA *ca, unsigned char *in);
int ASN1_ctprof_ctfile(CA *ca);

int ASN1_clprof_info(CA *ca,unsigned char *in);
int ASN1_clprof_stat(CA *ca, unsigned char *in);
int ASN1_prof_ext(CA *ca,unsigned char *in,unsigned char pf);

#define ASN1_ctprof_ext(ca,in)	ASN1_prof_ext((ca),(in),0xa3)
#define ASN1_clprof_ext(ca,in)	ASN1_prof_ext((ca),(in),0xa4)

int Prof_estimate_ctdersize(CertProf *cpf);
unsigned char *Prof_cert_toDER(CA *ca,unsigned char *buf,int *ret_len);
int Prof_DER_ctstat(CertProf *cpf,unsigned char *ret,int *ret_len);
int Prof_DER_ctctstat(CertProf *cpf,unsigned char *ret,int *ret_len);
int Prof_DER_ext(CA *ca,char exp,unsigned char *ret,int *ret_len);

#define Prof_DER_ctext(ca,ret,ret_len)	Prof_DER_ext((ca),3,(ret),(ret_len))
#define Prof_DER_clext(ca,ret,ret_len)	Prof_DER_ext((ca),4,(ret),(ret_len))

int Prof_estimate_cldersize(CRLProf *cpf);
unsigned char *Prof_crl_toDER(CA *ca,unsigned char *buf,int *ret_len);
int Prof_DER_clstat(CRLProf *cpf,unsigned char *ret,int *ret_len);


/* prof_file.c */
int Prof_open_ctinfo(CA *ca,char *path);
int Prof_save_ctinfo(CA *ca,char *path);
int Prof_open_ctfile(CA *ca, char *path);

int Prof_open_clinfo(CA *ca,char *path);
int Prof_save_clinfo(CA *ca,char *path);


/* prof_tool.c */
int Prof_now_certnum(CertProf *cpf);
int Prof_now_extnum(CertProf *cpf);
int Prof_expire_check(CertProf *cpf);
CertStat *Prof_find_stat(CertProf *cpf,int serial);
CertStat *Prof_find_statstr(CertProf *cpf,char *subject);
CertProf *Prof_find(CA *ca,char *name);
CRLProf *Prof_crl_find(CA *ca,char *name);

int Prof_add_cert(CertProf *cpf,Cert *ct);
void Prof_delete_stat(CertStat *st);
int Prof_add_certfile(CertProf *cpf,Cert * ct);
int Prof_add_xcertfile(CertProf *cpf,CertPair * cp);

int Prof_add_ext(CertProf *cpf,CertExt *et);
int Prof_delete_ext(CertProf *cpf,int id);


#ifdef  __cplusplus
}
#endif

#endif /* __OK_CA_H__ */
